@@ -115,6 +115,7 @@
 #include <stdio.h>
 #include <string.h>	       /* memcpy, memcmp */
 #include "types.h"             /* for byte and u32 typedefs */
+#include "util.h"
 #include "errors.h"
 #include "algorithms.h"
 
@@ -140,9 +141,9 @@ working_memcmp( const char *a, const char *b, size_t n )
 /* Some defines/checks to support standalone modules */
 
 #ifndef CIPHER_ALGO_3DES
-  #define CIPHER_ALGO_3DES 2
+#define CIPHER_ALGO_3DES 2
 #elif CIPHER_ALGO_3DES != 2
-  #error CIPHER_ALGO_3DES is defined to a wrong value.
+#error CIPHER_ALGO_3DES is defined to a wrong value.
 #endif
 
 
@@ -453,7 +454,7 @@ burn_stack (int bytes)
 {
     char buf[64];
     
-    memset (buf, 0, sizeof buf);
+    wipememory(buf,sizeof buf);
     bytes -= sizeof buf;
     if (bytes > 0)
         burn_stack (bytes);
@@ -1021,4 +1022,3 @@ des_get_info( int algo, size_t *keylen,
     }
     return NULL;
 }
-
