@@ -550,7 +550,7 @@ burn_stack (int bytes)
 {
     char buf[64];
     
-    memset (buf, 0, sizeof buf);
+    wipememory(buf,sizeof buf);
     bytes -= sizeof buf;
     if (bytes > 0)
         burn_stack (bytes);
@@ -1070,7 +1070,7 @@ twofish_get_info (int algo, size_t *keylen,
 
 #ifdef IS_MODULE
 static
-const char * const gnupgext_version = "TWOFISH ($Revision: 1.14 $)";
+const char * const gnupgext_version = "TWOFISH ($Revision: 1.14.2.1 $)";
 
 static struct {
     int class;
