@@ -1,5 +1,5 @@
 /* g10.c - The GnuPG utility (main for gpg)
- * Copyright (C) 1998,1999,2000,2001,2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998,1999,2000,2001,2002,2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -26,7 +26,7 @@
 #include <ctype.h>
 #include <unistd.h>
 #ifdef HAVE_DOSISH_SYSTEM
-  #include <fcntl.h> /* for setmode() */
+#include <fcntl.h> /* for setmode() */
 #endif
 #ifdef HAVE_STAT
 #include <sys/stat.h> /* for stat() */
@@ -69,7 +69,7 @@ enum cmd_and_opt_values { aNull = 0,
     oUser	  = 'u',
     oVerbose	  = 'v',
     oCompress	  = 'z',
-    oNotation	  = 'N',
+    oSetNotation  = 'N',
     oBatch	  = 500,
     oSigNotation,
     oCertNotation,
@@ -118,6 +118,7 @@ enum cmd_and_opt_values { aNull = 0,
     aListTrustDB,
     aListTrustPath,
     aExportOwnerTrust,
+    aListOwnerTrust,
     aImportOwnerTrust,
     aDeArmor,
     aEnArmor,
@@ -127,6 +128,7 @@ enum cmd_and_opt_values { aNull = 0,
     aRefreshKeys,
 
     oTextmode,
+    oNoTextmode,
     oExpert,
     oNoExpert,
     oAskSigExpire,
@@ -149,41 +151,33 @@ enum cmd_and_opt_values { aNull = 0,
     oDebug,
     oDebugAll,
     oStatusFD,
-#ifdef __riscos__
-    oStatusFile,
-#endif /* __riscos__ */
+    oStatusFile, /* only used with risc os */
     oAttributeFD,
-#ifdef __riscos__
-    oAttributeFile,
-#endif /* __riscos__ */
+    oAttributeFile, /* only used with risc os */
     oSKComments,
     oNoSKComments,
-    oNoVersion,
     oEmitVersion,
+    oNoEmitVersion,
     oCompletesNeeded,
     oMarginalsNeeded,
     oMaxCertDepth,
     oLoadExtension,
+    oGnuPG,
     oRFC1991,
+    oRFC2440,
     oOpenPGP,
     oPGP2,
-    oNoPGP2,
     oPGP6,
-    oNoPGP6,
     oPGP7,
-    oNoPGP7,
+    oPGP8,
     oCipherAlgo,
     oDigestAlgo,
     oCertDigestAlgo,
     oCompressAlgo,
     oPasswdFD,
-#ifdef __riscos__
-    oPasswdFile,
-#endif /* __riscos__ */
+    oPasswdFile, /* only used with risc os */
     oCommandFD,
-#ifdef __riscos__
-    oCommandFile,
-#endif /* __riscos__ */
+    oCommandFile, /* only used with risc os */
     oQuickRandom,
     oNoVerbose,
     oTrustDBName,
@@ -203,6 +197,7 @@ enum cmd_and_opt_values { aNull = 0,
     oCompressKeys,
     oCompressSigs,
     oAlwaysTrust,
+    oTrustModel,
     oEmuChecksumBug,
     oRunAsShmCP,
     oSetFilename,
@@ -248,9 +243,7 @@ enum cmd_and_opt_values { aNull = 0,
     oEncryptTo,
     oNoEncryptTo,
     oLoggerFD,
-#ifdef __riscos__
-    oLoggerFile,
-#endif /* __riscos__ */
+    oLoggerFile, /* only used with risc os */
     oUtf8Strings,
     oNoUtf8Strings,
     oDisableCipherAlgo,
@@ -299,6 +292,11 @@ enum cmd_and_opt_values { aNull = 0,
     oLCctype,
     oLCmessages,
     oGroup,
+    oStrict,
+    oNoStrict,
+    oMangleDosFilenames,
+    oNoMangleDosFilenames,
+    oEnableProgressFilter,                         
 aTest };
 
 
@@ -350,7 +348,7 @@ static ARGPARSE_OPTS opts[] = {
     { aExportOwnerTrust,
 	      "export-ownertrust", 256, N_("export the ownertrust values")},
     { aImportOwnerTrust,
-	      "import-ownertrust", 256 , N_("import ownertrust values")},
+	      "import-ownertrust", 256, N_("import ownertrust values")},
     { aUpdateTrustDB,
 	      "update-trustdb",0 , N_("update the trust database")},
     { aCheckTrustDB,
@@ -383,6 +381,7 @@ static ARGPARSE_OPTS opts[] = {
     { oCompress, NULL,	      1, N_("|N|set compress level N (0 disables)") },
     { oTextmodeShort, NULL,   0, "@"},
     { oTextmode, "textmode",  0, N_("use canonical text mode")},
+    { oNoTextmode, "no-textmode",  0, "@"},
     { oExpert, "expert",   0, "@"},
     { oNoExpert, "no-expert",   0, "@"},
     { oAskSigExpire, "ask-sig-expire",   0, "@"},
@@ -438,14 +437,18 @@ static ARGPARSE_OPTS opts[] = {
     { oMaxCertDepth,	"max-cert-depth", 1, "@" },
     { oTrustedKey, "trusted-key", 2, N_("|KEYID|ultimately trust this key")},
     { oLoadExtension, "load-extension" ,2, N_("|FILE|load extension module FILE")},
+    { oGnuPG, "gnupg",   0, "@"},
+    { oGnuPG, "no-pgp2", 0, "@"},
+    { oGnuPG, "no-pgp6", 0, "@"},
+    { oGnuPG, "no-pgp7", 0, "@"},
+    { oGnuPG, "no-pgp8", 0, "@"},
     { oRFC1991, "rfc1991",   0, N_("emulate the mode described in RFC1991")},
+    { oRFC2440, "rfc2440", 0, "@" },
     { oOpenPGP, "openpgp", 0, N_("set all packet, cipher and digest options to OpenPGP behavior")},
     { oPGP2, "pgp2", 0, N_("set all packet, cipher and digest options to PGP 2.x behavior")},
-    { oNoPGP2, "no-pgp2", 0, "@"},
     { oPGP6, "pgp6", 0, "@"},
-    { oNoPGP6, "no-pgp6", 0, "@"},
     { oPGP7, "pgp7", 0, "@"},
-    { oNoPGP7, "no-pgp7", 0, "@"},
+    { oPGP8, "pgp8", 0, "@"},
     { oS2KMode, "s2k-mode",  1, N_("|N|use passphrase mode N")},
     { oS2KDigest, "s2k-digest-algo",2,
 		N_("|NAME|use message digest algorithm NAME for passphrases")},
@@ -460,9 +463,10 @@ static ARGPARSE_OPTS opts[] = {
     { oShowPhotos,   "show-photos", 0, N_("Show Photo IDs")},
     { oNoShowPhotos, "no-show-photos", 0, N_("Don't show Photo IDs")},
     { oPhotoViewer,  "photo-viewer", 2, N_("Set command line to view Photo IDs")},
-    { oNotation,   "notation-data", 2, "@" },
-    { oSigNotation,   "sig-notation", 2, "@" },
-    { oCertNotation,  "cert-notation", 2, "@" },
+    { oSetNotation,  "set-notation", 2, "@" },
+    { oSetNotation,  "notation-data", 2, "@" }, /* Alias */
+    { oSigNotation,  "sig-notation", 2, "@" },
+    { oCertNotation, "cert-notation", 2, "@" },
 
     { 302, NULL, 0, N_(
   "@\n(See the man page for a complete listing of all commands and options)\n"
@@ -476,10 +480,12 @@ static ARGPARSE_OPTS opts[] = {
     " --fingerprint [names]      show fingerprints\n"  ) },
 
   /* hidden options */
-    { aExportOwnerTrust, "list-ownertrust",0 , "@"},  /* alias */
+    { aListOwnerTrust, "list-ownertrust", 256, "@"}, /* deprecated */
+    { oCompressAlgo, "compression-algo", 1, "@"}, /* alias */
     { aPrintMDs, "print-mds" , 256, "@"}, /* old */
     { aListTrustDB, "list-trustdb",0 , "@"},
-    { aListTrustPath, "list-trust-path",0, "@"},
+    /* Not yet used */
+    /* { aListTrustPath, "list-trust-path",0, "@"}, */
     { aPipeMode,  "pipemode", 0, "@" },
     { oKOption, NULL,	 0, "@"},
     { oPasswdFD, "passphrase-fd",1, "@" },
@@ -513,6 +519,7 @@ static ARGPARSE_OPTS opts[] = {
     { oCompressSigs, "compress-sigs",0, "@"},
     { oDefCertCheckLevel, "default-cert-check-level", 1, "@"},
     { oAlwaysTrust, "always-trust", 0, "@"},
+    { oTrustModel, "trust-model", 2, "@"},
     { oEmuChecksumBug, "emulate-checksum-bug", 0, "@"},
     { oRunAsShmCP, "run-as-shm-coprocess", 4, "@" },
     { oSetFilename, "set-filename", 2, "@" },
@@ -527,8 +534,9 @@ static ARGPARSE_OPTS opts[] = {
     { oNoShowNotation, "no-show-notation", 0, "@" },
     { oComment, "comment", 2, "@" },
     { oDefaultComment, "default-comment", 0, "@" },
-    { oNoVersion, "no-version", 0, "@"},
     { oEmitVersion, "emit-version", 0, "@"},
+    { oNoEmitVersion, "no-emit-version", 0, "@"},
+    { oNoEmitVersion, "no-version", 0, "@"}, /* alias */
     { oNotDashEscaped, "not-dash-escaped", 0, "@" },
     { oEscapeFrom, "escape-from-lines", 0, "@" },
     { oNoEscapeFrom, "no-escape-from-lines", 0, "@" },
@@ -587,6 +595,11 @@ static ARGPARSE_OPTS opts[] = {
     { oLCctype,    "lc-ctype",    2, "@" },
     { oLCmessages, "lc-messages", 2, "@" },
     { oGroup,      "group",       2, "@" },
+    { oStrict,     "strict",      0, "@" },
+    { oNoStrict,   "no-strict",   0, "@" },
+    { oMangleDosFilenames, "mangle-dos-filenames", 0, "@" },
+    { oNoMangleDosFilenames, "no-mangle-dos-filenames", 0, "@" },
+    { oEnableProgressFilter, "enable-progress-filter", 0, "@" },
 {0} };
 
 
@@ -600,7 +613,6 @@ static char *build_list( const char *text, char letter,
 			 const char *(*mapf)(int), int (*chkf)(int) );
 static void set_cmd( enum cmd_and_opt_values *ret_cmd,
 			enum cmd_and_opt_values new_cmd );
-static void print_hex( byte *p, size_t n );
 static void print_mds( const char *fname, int algo );
 static void add_notation_data( const char *string, int which );
 static void add_policy_url( const char *string, int which );
@@ -641,25 +653,25 @@ strusage( int level )
       case 33: p = _("\nSupported algorithms:\n"); break;
       case 34:
 	if( !pubkeys )
-	    pubkeys = build_list("Pubkey: ", 0, pubkey_algo_to_string,
+	    pubkeys = build_list(_("Pubkey: "), 0, pubkey_algo_to_string,
 							check_pubkey_algo );
 	p = pubkeys;
 	break;
       case 35:
 	if( !ciphers )
-	    ciphers = build_list("Cipher: ", 'S', cipher_algo_to_string,
+	    ciphers = build_list(_("Cipher: "), 'S', cipher_algo_to_string,
 							check_cipher_algo );
 	p = ciphers;
 	break;
       case 36:
 	if( !digests )
-	    digests = build_list("Hash: ", 'H', digest_algo_to_string,
+	    digests = build_list(_("Hash: "), 'H', digest_algo_to_string,
 							check_digest_algo );
 	p = digests;
 	break;
       case 37:
 	if( !zips )
-	    zips = build_list("Compress: ",'Z',compress_algo_to_string,
+	    zips = build_list(_("Compression: "),'Z',compress_algo_to_string,
 			                                check_compress_algo);
 	p = zips;
 	break;
@@ -728,15 +740,15 @@ build_list( const char *text, char letter,
 static void
 i18n_init(void)
 {
-  #ifdef USE_SIMPLE_GETTEXT
+#ifdef USE_SIMPLE_GETTEXT
     set_gettext_file( PACKAGE );
-  #else
-  #ifdef ENABLE_NLS
+#else
+#ifdef ENABLE_NLS
     setlocale( LC_ALL, "" );
     bindtextdomain( PACKAGE, G10_LOCALEDIR );
     textdomain( PACKAGE );
-  #endif
-  #endif
+#endif
+#endif
 }
 
 static void
@@ -832,9 +844,14 @@ static void add_group(char *string)
       return;
     }
 
+  trim_trailing_ws(name,strlen(name));
+
   /* Break apart the values */
-  while((value=strsep(&string," ")) && *value!='\0')
-    add_to_strlist2(&values,value,utf8_strings);
+  while ((value=strsep (&string," \t")))
+    {
+      if (*value)
+        add_to_strlist2(&values,value,utf8_strings);
+    }
 
   item=m_alloc(sizeof(struct groupitem));
   item->name=name;
@@ -1057,14 +1074,14 @@ main( int argc, char **argv )
     int pwfd = -1;
     int with_fpr = 0; /* make an option out of --fingerprint */
     int any_explicit_recipient = 0;
-  #ifdef USE_SHM_COPROCESSING
+#ifdef USE_SHM_COPROCESSING
     ulong requested_shm_size=0;
-  #endif
+#endif
 
-  #ifdef __riscos__
+#ifdef __riscos__
     riscos_global_defaults();
     opt.lock_once = 1;
-  #endif /* __riscos__ */
+#endif /* __riscos__ */
 
     trap_unaligned();
     secmem_set_flags( secmem_get_flags() | 2 ); /* suspend warnings */
@@ -1097,12 +1114,13 @@ main( int argc, char **argv )
     opt.import_options=0;
     opt.export_options=
       EXPORT_INCLUDE_NON_RFC|EXPORT_INCLUDE_ATTRIBUTES;
-    opt.keyserver_options.import_options=IMPORT_REPAIR_HKP_SUBKEY_BUG;
+    opt.keyserver_options.import_options=IMPORT_REPAIR_PKS_SUBKEY_BUG;
     opt.keyserver_options.export_options=
       EXPORT_INCLUDE_NON_RFC|EXPORT_INCLUDE_ATTRIBUTES;
     opt.keyserver_options.include_subkeys=1;
     opt.keyserver_options.include_revoked=1;
-#if defined (__MINGW32__)
+    opt.mangle_dos_filenames = 1;
+#if defined (_WIN32)
     set_homedir ( read_w32_registry_string( NULL,
                                     "Software\\GNU\\GnuPG", "HomeDir" ));
 #else
@@ -1132,7 +1150,7 @@ main( int argc, char **argv )
 	    set_homedir ( pargs.r.ret_str );
 	else if( pargs.r_opt == oNoPermissionWarn )
 	    opt.no_perm_warn=1;
-      #ifdef USE_SHM_COPROCESSING
+#ifdef USE_SHM_COPROCESSING
 	else if( pargs.r_opt == oRunAsShmCP ) {
 	    /* does not make sense in a options file, we do it here,
 	     * so that we are the able to drop setuid as soon as possible */
@@ -1144,7 +1162,7 @@ main( int argc, char **argv )
 	     * initialized when init_shm_coprocessing() is called */
 	    set_status_fd( iobuf_translate_file_handle (pargs.r.ret_int, 1) );
 	}
-      #endif
+#endif
     }
 
 #ifdef HAVE_DOSISH_SYSTEM
@@ -1163,7 +1181,7 @@ main( int argc, char **argv )
     }
 #endif
     /* initialize the secure memory. */
-    secmem_init( 16384 );
+    secmem_init( 32768 );
     maybe_setuid = 0;
     /* Okay, we are now working under our real uid */
 
@@ -1284,6 +1302,9 @@ main( int argc, char **argv )
 	  case aListTrustPath: set_cmd( &cmd, aListTrustPath); break;
 	  case aDeArmor: set_cmd( &cmd, aDeArmor); break;
 	  case aEnArmor: set_cmd( &cmd, aEnArmor); break;
+	  case aListOwnerTrust:
+	    deprecated_warning(configname,configlineno,
+			       "--list-ownertrust","--export-ownertrust","");
 	  case aExportOwnerTrust: set_cmd( &cmd, aExportOwnerTrust); break;
 	  case aImportOwnerTrust: set_cmd( &cmd, aImportOwnerTrust); break;
           case aPipeMode: set_cmd( &cmd, aPipeMode); break;
@@ -1364,8 +1385,8 @@ main( int argc, char **argv )
 	  case oQuickRandom: quick_random_gen(1); break;
 	  case oSKComments: opt.sk_comments=1; break;
 	  case oNoSKComments: opt.sk_comments=0; break;
-	  case oNoVersion: opt.no_version=1; break;
 	  case oEmitVersion: opt.no_version=0; break;
+	  case oNoEmitVersion: opt.no_version=1; break;
 	  case oCompletesNeeded: opt.completes_needed = pargs.r.ret_int; break;
 	  case oMarginalsNeeded: opt.marginals_needed = pargs.r.ret_int; break;
 	  case oMaxCertDepth: opt.max_cert_depth = pargs.r.ret_int; break;
@@ -1394,9 +1415,17 @@ main( int argc, char **argv )
 	  case oCompressKeys: opt.compress_keys = 1; break;
 	  case aListSecretKeys: set_cmd( &cmd, aListSecretKeys); break;
 	  case oAlwaysTrust: opt.always_trust = 1; break;
+	  case oTrustModel:
+	    if(ascii_strcasecmp(pargs.r.ret_str,"classic")==0)
+	      opt.always_trust=0;
+	    else if(ascii_strcasecmp(pargs.r.ret_str,"always")==0)
+	      opt.always_trust=1;
+	    else
+	      log_error("unknown trust model \"%s\"\n",pargs.r.ret_str);
+	    break;
 	  case oLoadExtension:
 #ifndef __riscos__
-#if defined(USE_DYNAMIC_LINKING) || defined(__MINGW32__)
+#if defined(USE_DYNAMIC_LINKING) || defined(_WIN32)
 	    if(check_permissions(pargs.r.ret_str,2))
 	      log_info(_("cipher extension \"%s\" not loaded due to "
 			 "unsafe permissions\n"),pargs.r.ret_str);
@@ -1409,17 +1438,16 @@ main( int argc, char **argv )
 #endif /* __riscos__ */
 	    break;
 	  case oRFC1991:
-	    opt.rfc1991 = 1;
-	    opt.rfc2440 = 0;
+	    opt.compliance = CO_RFC1991;
 	    opt.force_v4_certs = 0;
 	    opt.disable_mdc = 1;
 	    opt.escape_from = 1;
 	    break;
+	  case oRFC2440:
 	  case oOpenPGP:
 	    /* TODO: When 2440bis becomes a RFC, these may need
                changing. */
-	    opt.rfc1991 = 0;
-	    opt.rfc2440 = 1;
+	    opt.compliance = CO_RFC2440;
 	    opt.disable_mdc = 1;
 	    opt.allow_non_selfsigned_uid = 1;
 	    opt.allow_freeform_uid = 1;
@@ -1432,17 +1460,16 @@ main( int argc, char **argv )
 	    opt.def_cipher_algo = 0;
 	    opt.def_digest_algo = 0;
 	    opt.cert_digest_algo = 0;
-	    opt.def_compress_algo = 1;
+	    opt.def_compress_algo = -1;
             opt.s2k_mode = 3; /* iterated+salted */
 	    opt.s2k_digest_algo = DIGEST_ALGO_SHA1;
-	    opt.s2k_cipher_algo = CIPHER_ALGO_CAST5;
+	    opt.s2k_cipher_algo = CIPHER_ALGO_3DES;
 	    break;
-	  case oPGP2: opt.pgp2 = 1; break;
-	  case oNoPGP2: opt.pgp2 = 0; break;
-	  case oPGP6: opt.pgp6 = 1; break;
-	  case oNoPGP6: opt.pgp6 = 0; break;
-	  case oPGP7: opt.pgp7 = 1; break;
-	  case oNoPGP7: opt.pgp7 = 0; break;
+	  case oPGP2:  opt.compliance = CO_PGP2;  break;
+	  case oPGP6:  opt.compliance = CO_PGP6;  break;
+	  case oPGP7:  opt.compliance = CO_PGP7;  break;
+	  case oPGP8:  opt.compliance = CO_PGP8;  break;
+	  case oGnuPG: opt.compliance = CO_GNUPG; break;
 	  case oEmuMDEncodeBug: opt.emulate_bugs |= EMUBUG_MDENCODE; break;
 	  case oCompressSigs: opt.compress_sigs = 1; break;
 	  case oRunAsShmCP:
@@ -1497,6 +1524,7 @@ main( int argc, char **argv )
 	    break;
 	  case oTextmodeShort: opt.textmode = 2; break;
 	  case oTextmode: opt.textmode=1;  break;
+	  case oNoTextmode: opt.textmode=0;  break;
 	  case oExpert: opt.expert = 1; break;
 	  case oNoExpert: opt.expert = 0; break;
 	  case oAskSigExpire: opt.ask_sig_expire = 1; break;
@@ -1509,6 +1537,7 @@ main( int argc, char **argv )
 	  case oCompress: opt.compress = pargs.r.ret_int; break;
 	  case oPasswdFD:
             pwfd = iobuf_translate_file_handle (pargs.r.ret_int, 0);
+            opt.use_agent = 0;
             break;
 #ifdef __riscos__
 	  case oPasswdFile:
@@ -1581,7 +1610,7 @@ main( int argc, char **argv )
 	    else
 	      opt.exec_path_set=1;
 	    break;
-	  case oNotation:
+	  case oSetNotation:
 	    add_notation_data( pargs.r.ret_str, 0 );
 	    add_notation_data( pargs.r.ret_str, 1 );
 	    break;
@@ -1663,6 +1692,14 @@ main( int argc, char **argv )
           case oLCctype: opt.lc_ctype = pargs.r.ret_str; break;
           case oLCmessages: opt.lc_messages = pargs.r.ret_str; break;
 	  case oGroup: add_group(pargs.r.ret_str); break;
+	  case oStrict: /* noop */ break;
+	  case oNoStrict: /* noop */ break;
+
+          case oMangleDosFilenames: opt.mangle_dos_filenames = 1; break;
+          case oNoMangleDosFilenames: opt.mangle_dos_filenames = 0; break;
+
+        case oEnableProgressFilter: opt.enable_progress_filter = 1; break;
+
 	  default : pargs.err = configfp? 1:2; break;
 	}
     }
@@ -1684,13 +1721,13 @@ main( int argc, char **argv )
 			strusage(11), strusage(13), strusage(14) );
 	fprintf(stderr, "%s\n", strusage(15) );
     }
-  #ifdef IS_DEVELOPMENT_VERSION
+#ifdef IS_DEVELOPMENT_VERSION
     if( !opt.batch ) {
 	log_info("NOTE: THIS IS A DEVELOPMENT VERSION!\n");
 	log_info("It is only intended for test purposes and should NOT be\n");
 	log_info("used in a production environment or with production keys!\n");
     }
-  #endif
+#endif
 
     if (opt.verbose > 2)
         log_info ("using character set `%s'\n", get_native_charset ());
@@ -1727,105 +1764,98 @@ main( int argc, char **argv )
     set_debug();
 
     /* Do these after the switch(), so they can override settings. */
-    if(opt.pgp2 && (opt.pgp6 || opt.pgp7))
-      log_error(_("%s not allowed with %s!\n"),
-		"--pgp2",opt.pgp6?"--pgp6":"--pgp7");
-    else
+    if(PGP2)
       {
-	if(opt.pgp2)
+	int unusable=0;
+
+	if(cmd==aSign && !detached_sig)
 	  {
-	    int unusable=0;
+	    log_info(_("you can only make detached or clear signatures "
+		       "while in --pgp2 mode\n"));
+	    unusable=1;
+	  }
+	else if(cmd==aSignEncr || cmd==aSignSym)
+	  {
+	    log_info(_("you can't sign and encrypt at the "
+		       "same time while in --pgp2 mode\n"));
+	    unusable=1;
+	  }
+	else if(argc==0 && (cmd==aSign || cmd==aEncr || cmd==aSym))
+	  {
+	    log_info(_("you must use files (and not a pipe) when "
+		       "working with --pgp2 enabled.\n"));
+	    unusable=1;
+	  }
+	else if(cmd==aEncr || cmd==aSym)
+	  {
+	    /* Everything else should work without IDEA (except using
+	       a secret key encrypted with IDEA and setting an IDEA
+	       preference, but those have their own error
+	       messages). */
 
-	    if(cmd==aSign && !detached_sig)
+	    if(check_cipher_algo(CIPHER_ALGO_IDEA))
 	      {
-		log_info(_("you can only make detached or clear signatures "
-			   "while in --pgp2 mode\n"));
+		log_info(_("encrypting a message in --pgp2 mode requires "
+			   "the IDEA cipher\n"));
+		idea_cipher_warn(1);
 		unusable=1;
 	      }
-	    else if(cmd==aSignEncr || cmd==aSignSym)
+	    else if(cmd==aSym)
 	      {
-		log_info(_("you can't sign and encrypt at the "
-			   "same time while in --pgp2 mode\n"));
-		unusable=1;
-	      }
-	    else if(argc==0 && (cmd==aSign || cmd==aEncr || cmd==aSym))
-	      {
-		log_info(_("you must use files (and not a pipe) when "
-			   "working with --pgp2 enabled.\n"));
-		unusable=1;
-	      }
-	    else if(cmd==aEncr || cmd==aSym)
-	      {
-		/* Everything else should work without IDEA (except using
-		   a secret key encrypted with IDEA and setting an IDEA
-		   preference, but those have their own error
-		   messages). */
-
-		if(check_cipher_algo(CIPHER_ALGO_IDEA))
-		  {
-		    log_info(_("encrypting a message in --pgp2 mode requires "
-			       "the IDEA cipher\n"));
-		    idea_cipher_warn(1);
-		    unusable=1;
-		  }
-		else if(cmd==aSym)
-		  {
-		    /* This only sets IDEA for symmetric encryption
-		       since it is set via select_algo_from_prefs for
-		       pk encryption. */
-		    m_free(def_cipher_string);
-		    def_cipher_string = m_strdup("idea");
-		  }
-
-		/* PGP2 can't handle the output from the textmode
-		   filter, so we disable it for anything that could
-		   create a literal packet (only encryption and
-		   symmetric encryption, since we disable signing
-		   above). */
-		if(!unusable)
-		  opt.textmode=0;
+		/* This only sets IDEA for symmetric encryption
+		   since it is set via select_algo_from_prefs for
+		   pk encryption. */
+		m_free(def_cipher_string);
+		def_cipher_string = m_strdup("idea");
 	      }
 
-	    if(unusable)
-	      {
-		log_info(_("this message may not be usable by %s\n"),
-			 "PGP 2.x");
-		opt.pgp2=0;
-	      }
-	    else
-	      {
-		opt.rfc1991 = 1;
-		opt.rfc2440 = 0;
-		opt.force_mdc = 0;
-		opt.disable_mdc = 1;
-		opt.force_v4_certs = 0;
-		opt.sk_comments = 0;
-		opt.escape_from = 1;
-		opt.force_v3_sigs = 1;
-		opt.pgp2_workarounds = 1;
-		opt.ask_sig_expire = 0;
-		opt.ask_cert_expire = 0;
-		m_free(def_digest_string);
-		def_digest_string = m_strdup("md5");
-		opt.def_compress_algo = 1;
-	      }
+	    /* PGP2 can't handle the output from the textmode
+	       filter, so we disable it for anything that could
+	       create a literal packet (only encryption and
+	       symmetric encryption, since we disable signing
+	       above). */
+	    if(!unusable)
+	      opt.textmode=0;
 	  }
 
-	if(opt.pgp6 || opt.pgp7)
+	if(unusable)
+	  compliance_failure();
+	else
 	  {
-	    opt.sk_comments=0;
-	    opt.escape_from=1;
-	    opt.force_v3_sigs=1;
-	    opt.ask_sig_expire=0;
-	    opt.def_compress_algo=1;
-
-	    if(opt.pgp6) /* pgp7 has MDC */
-	      {
-		opt.force_mdc=0;
-		opt.disable_mdc=1;
-	      }
+	    opt.force_mdc = 0;
+	    opt.disable_mdc = 1;
+	    opt.force_v4_certs = 0;
+	    opt.sk_comments = 0;
+	    opt.escape_from = 1;
+	    opt.force_v3_sigs = 1;
+	    opt.pgp2_workarounds = 1;
+	    opt.ask_sig_expire = 0;
+	    opt.ask_cert_expire = 0;
+	    m_free(def_digest_string);
+	    def_digest_string = m_strdup("md5");
+	    opt.def_compress_algo = 1;
 	  }
       }
+    else if(PGP6)
+      {
+	opt.sk_comments=0;
+	opt.escape_from=1;
+	opt.force_v3_sigs=1;
+	opt.ask_sig_expire=0;
+	opt.force_mdc=0;
+	opt.disable_mdc=1;
+      }
+    else if(PGP7)
+      {
+	opt.sk_comments=0;
+	opt.escape_from=1;
+	opt.force_v3_sigs=1;
+	opt.ask_sig_expire=0;
+      }
+    else if(PGP8)
+      {
+	opt.escape_from=1;
+      }
 
     /* must do this after dropping setuid, because string_to...
      * may try to load an module */
@@ -1907,6 +1937,66 @@ main( int argc, char **argv )
     if( log_get_errorcount(0) )
 	g10_exit(2);
 
+    /* Check our chosen algorithms against the list of legal
+       algorithms. */
+
+    if(!GNUPG)
+      {
+	const char *badalg=NULL;
+	preftype_t badtype=PREFTYPE_NONE;
+
+	if(opt.def_cipher_algo
+	   && !algo_available(PREFTYPE_SYM,opt.def_cipher_algo,NULL))
+	  {
+	    badalg=cipher_algo_to_string(opt.def_cipher_algo);
+	    badtype=PREFTYPE_SYM;
+	  }
+	else if(opt.def_digest_algo
+		&& !algo_available(PREFTYPE_HASH,opt.def_digest_algo,NULL))
+	  {
+	    badalg=digest_algo_to_string(opt.def_digest_algo);
+	    badtype=PREFTYPE_HASH;
+	  }
+	else if(opt.cert_digest_algo
+		&& !algo_available(PREFTYPE_HASH,opt.cert_digest_algo,NULL))
+	  {
+	    badalg=digest_algo_to_string(opt.cert_digest_algo);
+	    badtype=PREFTYPE_HASH;
+	  }
+	else if(opt.def_compress_algo!=-1
+		&& !algo_available(PREFTYPE_ZIP,opt.def_compress_algo,NULL))
+	  {
+	    badalg=compress_algo_to_string(opt.def_compress_algo);
+	    badtype=PREFTYPE_ZIP;
+	  }
+
+	if(badalg)
+	  {
+	    switch(badtype)
+	      {
+	      case PREFTYPE_SYM:
+		log_info(_("you may not use cipher algorithm \"%s\" "
+			   "while in %s mode\n"),
+			 badalg,compliance_option_string());
+		break;
+	      case PREFTYPE_HASH:
+		log_info(_("you may not use digest algorithm \"%s\" "
+			   "while in %s mode\n"),
+			 badalg,compliance_option_string());
+		break;
+	      case PREFTYPE_ZIP:
+		log_info(_("you may not use compression algorithm \"%s\" "
+			   "while in %s mode\n"),
+			 badalg,compliance_option_string());
+		break;
+	      default:
+		BUG();
+	      }
+
+	    compliance_failure();
+	  }
+      }
+
     /* set the random seed file */
     if( use_random_seed ) {
 	char *p = make_filename(opt.homedir, "random_seed", NULL );
@@ -1983,13 +2073,6 @@ main( int argc, char **argv )
       case aEnArmor:
       case aFixTrustDB:
 	break;
-      case aKMode:
-      case aListKeys:
-      case aListSecretKeys:
-      case aCheckKeys:
-	if( opt.with_colons ) /* need this to list the trust */
-	    rc = setup_trustdb(1, trustdb_name );
-	break;
       case aExportOwnerTrust: rc = setup_trustdb( 0, trustdb_name ); break;
       case aListTrustDB: rc = setup_trustdb( argc? 1:0, trustdb_name ); break;
       default: rc = setup_trustdb(1, trustdb_name ); break;
@@ -2245,11 +2328,20 @@ main( int argc, char **argv )
 	for( ; argc; argc--, argv++ )
 	    add_to_strlist2( &sl, *argv, utf8_strings );
 	if( cmd == aSendKeys )
-	    keyserver_export( sl );
+	    rc=keyserver_export( sl );
 	else if( cmd == aRecvKeys )
-	    keyserver_import( sl );
+	    rc=keyserver_import( sl );
 	else
-	    export_pubkeys( sl, opt.export_options );
+	    rc=export_pubkeys( sl, opt.export_options );
+	if(rc)
+	  {
+	    if(cmd==aSendKeys)
+	      log_error(_("keyserver send failed: %s\n"),g10_errstr(rc));
+	    else if(cmd==aRecvKeys)
+	      log_error(_("keyserver receive failed: %s\n"),g10_errstr(rc));
+	    else
+	      log_error(_("key export failed: %s\n"),g10_errstr(rc));
+	  }
 	free_strlist(sl);
 	break;
 
@@ -2258,7 +2350,9 @@ main( int argc, char **argv )
 	for( ; argc; argc--, argv++ )
 	  append_to_strlist2( &sl, *argv, utf8_strings );
 
-	keyserver_search( sl );
+	rc=keyserver_search( sl );
+	if(rc)
+	  log_error(_("keyserver search failed: %s\n"),g10_errstr(rc));
 	free_strlist(sl);
 	break;
 
@@ -2266,7 +2360,9 @@ main( int argc, char **argv )
 	sl = NULL;
 	for( ; argc; argc--, argv++ )
 	    add_to_strlist2( &sl, *argv, utf8_strings );
-	keyserver_refresh(sl);
+	rc=keyserver_refresh(sl);
+	if(rc)
+	  log_error(_("keyserver refresh failed: %s\n"),g10_errstr(rc));
 	free_strlist(sl);
 	break;
 
@@ -2371,9 +2467,9 @@ main( int argc, char **argv )
 		size_t n = !endless && count < 99? count : 99;
 
 		p = get_random_bits( n*8, level, 0);
-	      #ifdef HAVE_DOSISH_SYSTEM
+#ifdef HAVE_DOSISH_SYSTEM
 		setmode ( fileno(stdout), O_BINARY );
-	      #endif
+#endif
                 if (opt.armor) {
                     char *tmp = make_radix64_string (p, n);
                     fputs (tmp, stdout);
@@ -2541,40 +2637,89 @@ g10_exit( int rc )
 }
 
 
-
-
+/* Pretty-print hex hashes.  This assumes at least an 80-character
+   display, but there are a few other similar assumptions in the
+   display code. */
 static void
-print_hex( byte *p, size_t n )
+print_hex( MD_HANDLE md, int algo, const char *fname )
 {
-    int i;
+  int i,n,count,indent=0;
+  const byte *p;
 
-    if( n == 20 ) {
-	for(i=0; i < n ; i++, i++, p += 2 ) {
-	    if( i )
-		putchar(' ');
-	    if( i == 10 )
-		putchar(' ');
-	    printf("%02X%02X", *p, p[1] );
-	}
+  if(fname)
+    indent=printf("%s: ",fname);
+
+  if(indent>40)
+    {
+      printf("\n");
+      indent=0;
     }
-    else if( n == 24 ) {
-	for(i=0; i < n ; i += 4, p += 4 ) {
-	    if( i )
-		putchar(' ');
-	    if( i == 12 )
-		putchar(' ');
-	    printf("%02X%02X%02X%02X", *p, p[1], p[2], p[3] );
+
+  if(algo==DIGEST_ALGO_RMD160)
+    indent+=printf("RMD160 = ");
+  else if(algo==DIGEST_ALGO_TIGER)
+    indent+=printf(" TIGER = ");
+  else if(algo>0)
+    indent+=printf("%6s = ",digest_algo_to_string(algo));
+  else
+    algo=abs(algo);
+
+  count=indent;
+
+  p = md_read( md, algo );
+  n = md_digest_length(algo);
+
+  count+=printf("%02X",*p++);
+
+  for(i=1;i<n;i++,p++)
+    {
+      if(n==16)
+	{
+	  if(count+2>79)
+	    {
+	      printf("\n%*s",indent," ");
+	      count=indent;
+	    }
+	  else
+	    count+=printf(" ");
+
+	  if(!(i%8))
+	    count+=printf(" ");
 	}
-    }
-    else {
-	for(i=0; i < n ; i++, p++ ) {
-	    if( i )
-		putchar(' ');
-	    if( i && !(i%8) )
-		putchar(' ');
-	    printf("%02X", *p );
+      else if (n==20)
+	{
+	  if(!(i%2))
+	    {
+	      if(count+4>79)
+		{
+		  printf("\n%*s",indent," ");
+		  count=indent;
+		}
+	      else
+		count+=printf(" ");
+	    }
+
+	  if(!(i%10))
+	    count+=printf(" ");
 	}
+      else
+	{
+	  if(!(i%4))
+	    {
+	      if(count+8>79)
+		{
+		  printf("\n%*s",indent," ");
+		  count=indent;
+		}
+	      else
+		count+=printf(" ");
+	    }
+	}
+
+      count+=printf("%02X",*p);
     }
+
+  printf("\n");
 }
 
 static void
@@ -2608,23 +2753,18 @@ print_mds( const char *fname, int algo )
     char buf[1024];
     size_t n;
     MD_HANDLE md;
-    char *pname;
 
     if( !fname ) {
 	fp = stdin;
-      #ifdef HAVE_DOSISH_SYSTEM
+#ifdef HAVE_DOSISH_SYSTEM
 	setmode ( fileno(fp) , O_BINARY );
-      #endif
-	pname = m_strdup("[stdin]: ");
+#endif
     }
     else {
-	pname = m_alloc(strlen(fname)+3);
-	strcpy(stpcpy(pname,fname),": ");
 	fp = fopen( fname, "rb" );
     }
     if( !fp ) {
-	log_error("%s%s\n", pname, strerror(errno) );
-	m_free(pname);
+	log_error("%s: %s\n", fname?fname:"[stdin]", strerror(errno) );
 	return;
     }
 
@@ -2637,12 +2777,17 @@ print_mds( const char *fname, int algo )
 	md_enable( md, DIGEST_ALGO_RMD160 );
 	if( !check_digest_algo(DIGEST_ALGO_TIGER) )
 	    md_enable( md, DIGEST_ALGO_TIGER );
+	md_enable( md, DIGEST_ALGO_SHA256 );
+	if( !check_digest_algo(DIGEST_ALGO_SHA384) )
+	  md_enable( md, DIGEST_ALGO_SHA384 );
+	if( !check_digest_algo(DIGEST_ALGO_SHA512) )
+	  md_enable( md, DIGEST_ALGO_SHA512 );
     }
 
     while( (n=fread( buf, 1, DIM(buf), fp )) )
 	md_write( md, buf, n );
     if( ferror(fp) )
-	log_error("%s%s\n", pname, strerror(errno) );
+	log_error("%s: %s\n", fname?fname:"[stdin]", strerror(errno) );
     else {
 	md_final(md);
         if ( opt.with_colons ) {
@@ -2654,27 +2799,28 @@ print_mds( const char *fname, int algo )
                 print_hashline( md, DIGEST_ALGO_RMD160, fname );
                 if( !check_digest_algo(DIGEST_ALGO_TIGER) ) 
                     print_hashline( md, DIGEST_ALGO_TIGER, fname );
+                print_hashline( md, DIGEST_ALGO_SHA256, fname );
+                if( !check_digest_algo(DIGEST_ALGO_SHA384) ) 
+		  print_hashline( md, DIGEST_ALGO_SHA384, fname );
+                if( !check_digest_algo(DIGEST_ALGO_SHA512) ) 
+		  print_hashline( md, DIGEST_ALGO_SHA512, fname );
             }
         }
         else {
-            if( algo ) {
-                if( fname )
-                    fputs( pname, stdout );
-                print_hex(md_read(md, algo), md_digest_length(algo) );
-            }
+            if( algo )
+	       print_hex(md,-algo,fname);
             else {
-                printf(  "%s   MD5 = ", fname?pname:"" );
-                print_hex(md_read(md, DIGEST_ALGO_MD5), 16 );
-                printf("\n%s  SHA1 = ", fname?pname:""  );
-                print_hex(md_read(md, DIGEST_ALGO_SHA1), 20 );
-                printf("\n%sRMD160 = ", fname?pname:""  );
-                print_hex(md_read(md, DIGEST_ALGO_RMD160), 20 );
-                if( !check_digest_algo(DIGEST_ALGO_TIGER) ) {
-                    printf("\n%s TIGER = ", fname?pname:""  );
-                    print_hex(md_read(md, DIGEST_ALGO_TIGER), 24 );
-                }
+                print_hex( md, DIGEST_ALGO_MD5, fname );
+                print_hex( md, DIGEST_ALGO_SHA1, fname );
+                print_hex( md, DIGEST_ALGO_RMD160, fname );
+                if( !check_digest_algo(DIGEST_ALGO_TIGER) )
+		  print_hex( md, DIGEST_ALGO_TIGER, fname );
+                print_hex( md, DIGEST_ALGO_SHA256, fname );
+                if( !check_digest_algo(DIGEST_ALGO_SHA384) )
+		  print_hex( md, DIGEST_ALGO_SHA384, fname );
+                if( !check_digest_algo(DIGEST_ALGO_SHA512) )
+		  print_hex( md, DIGEST_ALGO_SHA512, fname );
             }
-            putchar('\n');
         }
     }
     md_close(md);
@@ -2696,6 +2842,7 @@ add_notation_data( const char *string, int which )
     STRLIST sl,*notation_data;
     int critical=0;
     int highbit=0;
+    int saw_at=0;
 
     if(which)
       notation_data=&opt.cert_notation_data;
@@ -2707,23 +2854,39 @@ add_notation_data( const char *string, int which )
 	string++;
     }
 
-    for( s=string ; *s != '='; s++ ) {
-	if( !*s || (*s & 0x80) || (!isgraph(*s) && !isspace(*s)) ) {
+    /* If and when the IETF assigns some official name tags, we'll
+       have to add them here. */
+
+    for( s=string ; *s != '='; s++ )
+      {
+	if( *s=='@')
+	  saw_at=1;
+
+	if( !*s || (*s & 0x80) || (!isgraph(*s) && !isspace(*s)) )
+	  {
 	    log_error(_("a notation name must have only printable characters "
 			"or spaces, and end with an '='\n") );
 	    return;
-	}
-    }
+	  }
+      }
+
+    if(!saw_at && !opt.expert)
+      {
+	log_error(
+	        _("a user notation name must contain the '@' character\n"));
+	return;
+      }
+
     /* we only support printable text - therefore we enforce the use
      * of only printable characters (an empty value is valid) */
     for( s++; *s ; s++ ) {
-	if( iscntrl(*s) ) {
+	if ((*s & 0x80))
+          highbit = 1;
+	else if (iscntrl(*s)) {
 	    log_error(_("a notation value must not use "
 			"any control characters\n") );
 	    return;
 	}
-	else if( *s & 0x80 )
-	    highbit = 1;
     }
 
     if( highbit )   /* must use UTF8 encoding */
