@@ -26,7 +26,7 @@
 #include <ctype.h>
 #include <unistd.h>
 #ifdef HAVE_DOSISH_SYSTEM
-  #include <fcntl.h> /* for setmode() */
+#include <fcntl.h> /* for setmode() */
 #endif
 
 #define INCLUDED_BY_MAIN_MODULE 1
@@ -125,20 +125,20 @@ set_homedir (char *dir)
 static void
 i18n_init(void)
 {
-  #ifdef USE_SIMPLE_GETTEXT
+#ifdef USE_SIMPLE_GETTEXT
     set_gettext_file( PACKAGE );
-  #else
-  #ifdef ENABLE_NLS
-    #ifdef HAVE_LC_MESSAGES
+#else
+#ifdef ENABLE_NLS
+#ifdef HAVE_LC_MESSAGES
        setlocale( LC_TIME, "" );
        setlocale( LC_MESSAGES, "" );
-    #else
+#else
        setlocale( LC_ALL, "" );
-    #endif
+#endif
     bindtextdomain( PACKAGE, G10_LOCALEDIR );
     textdomain( PACKAGE );
-  #endif
-  #endif
+#endif
+#endif
 }
 
 
@@ -151,9 +151,9 @@ main( int argc, char **argv )
     STRLIST nrings=NULL;
     unsigned configlineno;
 
-  #ifdef __riscos__
+#ifdef __riscos__
     riscos_global_defaults();
-  #endif /* __riscos__ */
+#endif /* __riscos__ */
 
     log_set_name("gpgv");
     init_signals();
@@ -190,6 +190,7 @@ main( int argc, char **argv )
 	  case oStatusFD: set_status_fd( pargs.r.ret_int ); break;
 	  case oLoggerFD: log_set_logfile( NULL, pargs.r.ret_int ); break;
 	  case oHomedir: set_homedir (pargs.r.ret_str); break;
+	  case oIgnoreTimeConflict: opt.ignore_time_conflict = 1; break;
 	  default : pargs.err = 2; break;
 	}
     }
@@ -235,13 +236,22 @@ check_signatures_trust( PKT_signature *sig )
     return 0;
 }
 
+void
+read_trust_options(byte *trust_model,ulong *created,ulong *nextcheck) {}
 
 /* Stub: 
  * We don't have the trustdb , so we have to provide some stub functions
  * instead
  */
+
 int
-get_validity_info (PKT_public_key *pk, const byte *namehash )
+cache_disabled_value(PKT_public_key *pk)
+{
+  return 0;
+}
+
+int
+get_validity_info (PKT_public_key *pk, PKT_user_id *uid)
 {
     return '?';
 }
@@ -315,9 +325,11 @@ check_secret_key( PKT_secret_key *sk, int n )
 DEK *
 passphrase_to_dek( u32 *keyid, int pubkey_algo,
 		   int cipher_algo, STRING2KEY *s2k, int mode,
-                   const char *tmp)
+                   const char *tmp, int *canceled)
 {
-    return NULL;
+  if (canceled)
+    *canceled = 0;
+  return NULL;
 }
 
 /* Stubs to avoid linking to photoid.c */
