@@ -1,5 +1,6 @@
 /* import.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003
+ *               Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -60,7 +61,6 @@ static int import( IOBUF inp, int fast, const char* fname,
                    struct stats_s *stats, unsigned int options );
 static int read_block( IOBUF a, PACKET **pending_pkt, KBNODE *ret_root );
 static void revocation_present(KBNODE keyblock);
-static void remove_bad_stuff (KBNODE keyblock);
 static int import_one( const char *fname, KBNODE keyblock, int fast,
                        struct stats_s *stats, unsigned int options);
 static int import_secret_one( const char *fname, KBNODE keyblock,
@@ -87,47 +87,15 @@ static int merge_keysigs( KBNODE dst, KBNODE src, int *n_sigs,
 int
 parse_import_options(char *str,unsigned int *options)
 {
-  char *tok;
-  int hit=0;
-  struct
-  {
-    char *name;
-    unsigned int bit;
-  } import_opts[]=
+  struct parse_options import_opts[]=
     {
       {"allow-local-sigs",IMPORT_ALLOW_LOCAL_SIGS},
-      {"repair-hkp-subkey-bug",IMPORT_REPAIR_HKP_SUBKEY_BUG},
+      {"repair-hkp-subkey-bug",IMPORT_REPAIR_PKS_SUBKEY_BUG},
+      {"repair-pks-subkey-bug",IMPORT_REPAIR_PKS_SUBKEY_BUG},
       {NULL,0}
     };
 
-  while((tok=strsep(&str," ,")))
-    {
-      int i,rev=0;
-
-      if(ascii_strncasecmp("no-",tok,3)==0)
-	{
-	  rev=1;
-	  tok+=3;
-	}
-
-      for(i=0;import_opts[i].name;i++)
-	{
-	  if(ascii_strcasecmp(import_opts[i].name,tok)==0)
-	    {
-	      if(rev)
-		*options&=~import_opts[i].bit;
-	      else
-		*options|=import_opts[i].bit;
-	      hit=1;
-	      break;
-	    }
-	}
-
-      if(!hit && !import_opts[i].name)
-	return 0;
-    }
-
-  return hit;
+  return parse_options(str,options,import_opts);
 }
 
 void *
@@ -196,6 +164,8 @@ import_keys( char **fnames, int nnames, int fast,
 	else {
 	    int rc = import( inp, fast, fname, stats, options );
 	    iobuf_close(inp);
+            /* must invalidate that ugly cache to actually close it */
+            iobuf_ioctl (NULL, 2, 0, (char*)fname);
 	    if( rc )
 		log_error("import from `%s' failed: %s\n", fname,
 							   g10_errstr(rc) );
@@ -246,7 +216,6 @@ import( IOBUF inp, int fast, const char* fname,
     }
 
     while( !(rc = read_block( inp, &pending_pkt, &keyblock) )) {
-        remove_bad_stuff (keyblock);
 	if( keyblock->pkt->pkttype == PKT_PUBLIC_KEY )
 	    rc = import_one( fname, keyblock, fast, stats, options );
 	else if( keyblock->pkt->pkttype == PKT_SECRET_KEY ) 
@@ -436,23 +405,7 @@ read_block( IOBUF a, PACKET **pending_pkt, KBNODE *ret_root )
     return rc;
 }
 
-
-static void
-remove_bad_stuff (KBNODE keyblock)
-{
-    KBNODE node;
-
-    for (node=keyblock; node; node = node->next ) {
-        if( node->pkt->pkttype == PKT_SIGNATURE ) {
-            /* delete the subpackets we used to use for the
-               verification cache */
-            delete_sig_subpkt (node->pkt->pkt.signature->unhashed,
-                               SIGSUBPKT_PRIV_VERIFY_CACHE);
-        }
-    }
-}
-
-/* Walk through the subkeys on a pk to find if we have the HKP
+/* Walk through the subkeys on a pk to find if we have the PKS
    disease: multiple subkeys with their binding sigs stripped, and the
    sig for the first subkey placed after the last subkey.  That is,
    instead of "pk uid sig sub1 bind1 sub2 bind2 sub3 bind3" we have
@@ -462,7 +415,7 @@ remove_bad_stuff (KBNODE keyblock)
    sub2 sub3".  Returns TRUE if the keyblock was modified. */
 
 static int
-fix_hkp_corruption(KBNODE keyblock)
+fix_pks_corruption(KBNODE keyblock)
 {
   int changed=0,keycount=0;
   KBNODE node,last=NULL,sknode=NULL;
@@ -493,6 +446,10 @@ fix_hkp_corruption(KBNODE keyblock)
 	  sknode->next=node;
 	  last->next=NULL;
 
+	  /* Note we aren't checking whether this binding sig is a
+	     selfsig.  This is not necessary here as the subkey and
+	     binding sig will be rejected later if that is the
+	     case. */
 	  if(check_key_signature(keyblock,node,NULL))
 	    {
 	      /* Not a match, so undo the changes. */
@@ -539,6 +496,29 @@ print_import_ok (PKT_public_key *pk, PKT_secret_key *sk, unsigned int reason)
 }
 
 
+void
+print_import_check (PKT_public_key * pk, PKT_user_id * id)
+{
+    char * buf;
+    byte fpr[24];
+    u32 keyid[2];
+    size_t i, pos = 0, n;
+
+    buf = m_alloc (17+41+id->len+32);
+    keyid_from_pk (pk, keyid);
+    sprintf (buf, "%08X%08X ", keyid[0], keyid[1]);
+    pos = 17;
+    fingerprint_from_pk (pk, fpr, &n);
+    for (i = 0; i < n; i++, pos += 2)
+        sprintf (buf+pos, "%02X", fpr[i]);
+    strcat (buf, " ");
+    pos += 1;
+    strcat (buf, id->name);
+    write_status_text (STATUS_IMPORT_CHECK, buf);
+    m_free (buf);
+}
+
+
 /****************
  * Try to import one keyblock.	Return an error only in serious cases, but
  * never for an invalid keyblock.  It uses log_error to increase the
@@ -567,7 +547,11 @@ import_one( const char *fname, KBNODE keyblock, int fast,
     keyid_from_pk( pk, keyid );
     uidnode = find_next_kbnode( keyblock, PKT_USER_ID );
 
-    if( opt.verbose ) {
+    if(pk->pubkey_algo==PUBKEY_ALGO_ELGAMAL)
+      log_info(_("NOTE: Elgamal primary key detected - "
+		 "this may take some time to import\n"));
+
+    if( opt.verbose && !opt.interactive ) {
 	log_info( "pub  %4u%c/%08lX %s   ",
 		  nbits_from_pk( pk ),
 		  pubkey_letter( pk->pubkey_algo ),
@@ -583,6 +567,9 @@ import_one( const char *fname, KBNODE keyblock, int fast,
     }
     
     if (opt.interactive) {
+        if(is_status_enabled())
+	  print_import_check (pk, uidnode->pkt->pkt.user_id);
+	merge_keys_and_selfsig (keyblock);
         tty_printf ("\n");
         show_basic_key_info (keyblock);
         tty_printf ("\n");
@@ -593,7 +580,10 @@ import_one( const char *fname, KBNODE keyblock, int fast,
 
     clear_kbnode_flags( keyblock );
 
-    if((options&IMPORT_REPAIR_HKP_SUBKEY_BUG) && fix_hkp_corruption(keyblock))
+    /* It's really PKS corruption, not HKP corruption, but I won't
+       change the string in stable. */
+    if((options&IMPORT_REPAIR_PKS_SUBKEY_BUG) && fix_pks_corruption(keyblock)
+       && opt.verbose)
       log_info(_("key %08lX: HKP subkey corruption repaired\n"),
 	       (ulong)keyid[1]);
 
@@ -615,18 +605,16 @@ import_one( const char *fname, KBNODE keyblock, int fast,
 	  }
 
     if( !delete_inv_parts( fname, keyblock, keyid, options ) ) {
-	if( !opt.quiet ) {
-	    log_info( _("key %08lX: no valid user IDs\n"),
-							(ulong)keyid[1]);
-	    log_info(_("this may be caused by a missing self-signature\n"));
-	}
+        log_error( _("key %08lX: no valid user IDs\n"), (ulong)keyid[1]);
+	if( !opt.quiet )
+	  log_info(_("this may be caused by a missing self-signature\n"));
 	stats->no_user_id++;
 	return 0;
     }
 
     /* do we have this key already in one of our pubrings ? */
     pk_orig = m_alloc_clear( sizeof *pk_orig );
-    rc = get_pubkey( pk_orig, keyid );
+    rc = get_pubkey_fast ( pk_orig, keyid );
     if( rc && rc != G10ERR_NO_PUBKEY && rc != G10ERR_UNU_PUBKEY ) {
 	log_error( _("key %08lX: public key not found: %s\n"),
 				(ulong)keyid[1], g10_errstr(rc));
@@ -909,8 +897,8 @@ import_revoke_cert( const char *fname, KBNODE node, struct stats_s *stats )
     pk = m_alloc_clear( sizeof *pk );
     rc = get_pubkey( pk, keyid );
     if( rc == G10ERR_NO_PUBKEY ) {
-	log_info( _("key %08lX: no public key - "
-		 "can't apply revocation certificate\n"), (ulong)keyid[1]);
+	log_error( _("key %08lX: no public key - "
+		     "can't apply revocation certificate\n"), (ulong)keyid[1]);
 	rc = 0;
 	goto leave;
     }
@@ -960,12 +948,12 @@ import_revoke_cert( const char *fname, KBNODE node, struct stats_s *stats )
 	if( onode->pkt->pkttype == PKT_USER_ID )
 	    break;
 	else if( onode->pkt->pkttype == PKT_SIGNATURE
-		 && onode->pkt->pkt.signature->sig_class == 0x20
-		 && keyid[0] == onode->pkt->pkt.signature->keyid[0]
-		 && keyid[1] == onode->pkt->pkt.signature->keyid[1] ) {
+		 && !cmp_signatures(node->pkt->pkt.signature,
+				    onode->pkt->pkt.signature))
+	  {
 	    rc = 0;
 	    goto leave; /* yes, we already know about it */
-	}
+	  }
     }
 
 
@@ -1029,11 +1017,18 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
 	  rsdate=0;
 	  bsnode=NULL;
 	  rsnode=NULL;
+	  continue;
 	}
       else if( n->pkt->pkttype != PKT_SIGNATURE )
 	    continue;
 	sig = n->pkt->pkt.signature;
 	if( keyid[0] == sig->keyid[0] && keyid[1] == sig->keyid[1] ) {
+
+	  /* This just caches the sigs for later use.  That way we
+	     import a fully-cached key which speeds things up. */
+	  if(!opt.no_sig_cache)
+	    check_key_signature(keyblock,n,NULL);
+
 	    if( (sig->sig_class&~3) == 0x10 ) {
 		KBNODE unode = find_prev_kbnode( keyblock, n, PKT_USER_ID );
 		if( !unode )  {
@@ -1047,15 +1042,18 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
 		  rc = check_key_signature( keyblock, n, NULL);
 		  if( rc )
 		    {
-		      char *p=utf8_to_native(unode->pkt->pkt.user_id->name,
+		      if( opt.verbose )
+			{
+			  char *p=utf8_to_native(unode->pkt->pkt.user_id->name,
 				      strlen(unode->pkt->pkt.user_id->name),0);
-		      log_info( rc == G10ERR_PUBKEY_ALGO ?
-				_("key %08lX: unsupported public key "
-				  "algorithm on user id \"%s\"\n"):
-				_("key %08lX: invalid self-signature "
-				  "on user id \"%s\"\n"),
-				(ulong)keyid[1],p);
-		      m_free(p);
+			  log_info( rc == G10ERR_PUBKEY_ALGO ?
+				    _("key %08lX: unsupported public key "
+				      "algorithm on user id \"%s\"\n"):
+				    _("key %08lX: invalid self-signature "
+				      "on user id \"%s\"\n"),
+				    (ulong)keyid[1],p);
+			  m_free(p);
+			}
 		    }
 		  else
 		    unode->flag |= 1; /* mark that signature checked */
@@ -1067,17 +1065,19 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
 		 revocation targets, this may need to be revised. */
 
 		if( !knode ) {
-		    log_info( _("key %08lX: no subkey for key binding\n"),
-					    (ulong)keyid[1]);
+		    if(opt.verbose)
+		      log_info( _("key %08lX: no subkey for key binding\n"),
+				(ulong)keyid[1]);
 		    n->flag |= 4; /* delete this */
 		}
 		else {
 		  rc = check_key_signature( keyblock, n, NULL);
 		  if( rc ) {
-		    log_info(  rc == G10ERR_PUBKEY_ALGO ?
+		    if(opt.verbose)
+		      log_info(rc == G10ERR_PUBKEY_ALGO ?
 			    _("key %08lX: unsupported public key algorithm\n"):
-		            _("key %08lX: invalid subkey binding\n"),
-		            (ulong)keyid[1]);
+			    _("key %08lX: invalid subkey binding\n"),
+			    (ulong)keyid[1]);
 		    n->flag|=4;
 		  }
 		  else {
@@ -1088,8 +1088,9 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
 			bsnode->flag|=4; /* Delete the last binding
 					    sig since this one is
 					    newer */
-			log_info(_("key %08lX: removed multiple subkey "
-				   "binding\n"),(ulong)keyid[1]);
+			if(opt.verbose)
+			  log_info(_("key %08lX: removed multiple subkey "
+				     "binding\n"),(ulong)keyid[1]);
 		      }
 
 		      bsnode=n;
@@ -1108,18 +1109,21 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
                  See the comment in getkey.c:merge_selfsigs_subkey for
                  more */
 		if( !knode ) {
-		    log_info( _("key %08lX: no subkey for key revocation\n"),
-					    (ulong)keyid[1]);
+		    if(opt.verbose)
+		      log_info( _("key %08lX: no subkey for key revocation\n"),
+				(ulong)keyid[1]);
 		    n->flag |= 4; /* delete this */
 		}
 		else {
 		  rc = check_key_signature( keyblock, n, NULL);
 		  if( rc ) {
-		    log_info(  rc == G10ERR_PUBKEY_ALGO ?
+		      if(opt.verbose)
+			log_info(rc == G10ERR_PUBKEY_ALGO ?
 			    _("key %08lX: unsupported public key algorithm\n"):
-		            _("key %08lX: invalid subkey revocation\n"),
-			       (ulong)keyid[1]);
-		    n->flag|=4;
+			    _("key %08lX: invalid subkey revocation\n"),
+			    (ulong)keyid[1]);
+
+		      n->flag|=4;
 		  }
 		  else {
 		    /* It's valid, so is it newer? */
@@ -1128,8 +1132,9 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
 			rsnode->flag|=4; /* Delete the last revocation
 					    sig since this one is
 					    newer */
-			log_info(_("key %08lX: removed multiple subkey "
-				   "revocation\n"),(ulong)keyid[1]);
+			if(opt.verbose)
+			  log_info(_("key %08lX: removed multiple subkey "
+				     "revocation\n"),(ulong)keyid[1]);
 		      }
 
 		      rsnode=n;
@@ -1214,18 +1219,18 @@ delete_inv_parts( const char *fname, KBNODE keyblock,
 	     * to import non-exportable signature when we have the
 	     * the secret key used to create this signature - it
 	     * seems that this makes sense */
-	    log_info( _("key %08lX: non exportable signature "
-				    "(class %02x) - skipped\n"),
-				    (ulong)keyid[1],
-				     node->pkt->pkt.signature->sig_class );
+	    if(opt.verbose)
+	      log_info( _("key %08lX: non exportable signature "
+			  "(class %02x) - skipped\n"),
+			(ulong)keyid[1], node->pkt->pkt.signature->sig_class );
 	    delete_kbnode( node );
 	}
 	else if( node->pkt->pkttype == PKT_SIGNATURE
 		 && node->pkt->pkt.signature->sig_class == 0x20 )  {
 	    if( uid_seen ) {
-		log_error( _("key %08lX: revocation certificate "
-				     "at wrong place - skipped\n"),
-				    (ulong)keyid[1]);
+	        if(opt.verbose)
+		  log_info( _("key %08lX: revocation certificate "
+			      "at wrong place - skipped\n"), (ulong)keyid[1]);
 		delete_kbnode( node );
 	    }
 	    else {
@@ -1240,9 +1245,10 @@ delete_inv_parts( const char *fname, KBNODE keyblock,
 		  int rc = check_key_signature( keyblock, node, NULL);
 		  if( rc )
 		    {
-		      log_error( _("key %08lX: invalid revocation "
-				   "certificate: %s - skipped\n"),
-				 (ulong)keyid[1], g10_errstr(rc));
+		      if(opt.verbose)
+			log_info( _("key %08lX: invalid revocation "
+				    "certificate: %s - skipped\n"),
+				  (ulong)keyid[1], g10_errstr(rc));
 		      delete_kbnode( node );
 		    }
 		}
@@ -1252,11 +1258,20 @@ delete_inv_parts( const char *fname, KBNODE keyblock,
 		 (node->pkt->pkt.signature->sig_class == 0x18 ||
 		  node->pkt->pkt.signature->sig_class == 0x28) &&
 		 !subkey_seen ) {
-	    log_error( _("key %08lX: subkey signature "
-			 "in wrong place - skipped\n"),
-		       (ulong)keyid[1]);
+	    if(opt.verbose)
+	      log_info( _("key %08lX: subkey signature "
+			  "in wrong place - skipped\n"), (ulong)keyid[1]);
 	    delete_kbnode( node );
 	}
+	else if( node->pkt->pkttype == PKT_SIGNATURE
+		 && !IS_CERT(node->pkt->pkt.signature))
+	  {
+	    if(opt.verbose)
+	      log_info(_("key %08lX: unexpected signature class (0x%02X) -"
+			 " skipped\n"),(ulong)keyid[1],
+		       node->pkt->pkt.signature->sig_class);
+	    delete_kbnode(node);
+	  }
 	else if( (node->flag & 4) ) /* marked for deletion */
 	    delete_kbnode( node );
     }
@@ -1350,8 +1365,9 @@ collapse_uids( KBNODE *keyblock )
 	kid1 = keyid_from_sk( n->pkt->pkt.secret_key, NULL );
     else
 	kid1 = 0;
-    log_info(_("key %08lX: duplicated user ID detected - merged\n"),
-								 (ulong)kid1);
+    if(!opt.quiet)
+      log_info(_("key %08lX: duplicated user ID detected - merged\n"),
+	       (ulong)kid1);
 
     return 1;
 }
@@ -1401,8 +1417,8 @@ revocation_present(KBNODE keyblock)
                          itself? */
 		      int rc;
 
-		      rc=get_pubkey_byfprint(NULL,sig->revkey[idx]->fpr,
-					     MAX_FINGERPRINT_LEN);
+		      rc=get_pubkey_byfprint_fast (NULL,sig->revkey[idx]->fpr,
+                                                   MAX_FINGERPRINT_LEN);
 		      if(rc==G10ERR_NO_PUBKEY || rc==G10ERR_UNU_PUBKEY)
 			{
 			  /* No, so try and get it */
@@ -1417,7 +1433,7 @@ revocation_present(KBNODE keyblock)
 						      MAX_FINGERPRINT_LEN);
 
 			      /* Do we have it now? */
-			      rc=get_pubkey_byfprint(NULL,
+			      rc=get_pubkey_byfprint_fast (NULL,
 						     sig->revkey[idx]->fpr,
 						     MAX_FINGERPRINT_LEN);
 			    }
@@ -1466,23 +1482,25 @@ merge_blocks( const char *fname, KBNODE keyblock_orig, KBNODE keyblock,
 		    break;
 		else if( onode->pkt->pkttype == PKT_SIGNATURE
 			 && onode->pkt->pkt.signature->sig_class == 0x20
-			 && node->pkt->pkt.signature->keyid[0]
-			    == onode->pkt->pkt.signature->keyid[0]
-			 && node->pkt->pkt.signature->keyid[1]
-			    == onode->pkt->pkt.signature->keyid[1] ) {
+			 && !cmp_signatures(onode->pkt->pkt.signature,
+					    node->pkt->pkt.signature))
+		  {
 		    found = 1;
 		    break;
-		}
+		  }
 	    }
 	    if( !found ) {
-	        char *p=get_user_id_printable (keyid);
 		KBNODE n2 = clone_kbnode(node);
 		insert_kbnode( keyblock_orig, n2, 0 );
 		n2->flag |= 1;
                 ++*n_sigs;
-		log_info(_("key %08lX: \"%s\" revocation certificate added\n"),
-					 (ulong)keyid[1],p);
-		m_free(p);
+		if(!opt.quiet)
+		  {
+		    char *p=get_user_id_printable (keyid);
+		    log_info(_("key %08lX: \"%s\" revocation "
+			       "certificate added\n"), (ulong)keyid[1],p);
+		    m_free(p);
+		  }
 	    }
 	}
     }
@@ -1511,8 +1529,9 @@ merge_blocks( const char *fname, KBNODE keyblock_orig, KBNODE keyblock,
 		insert_kbnode( keyblock_orig, n2, 0 );
 		n2->flag |= 1;
                 ++*n_sigs;
-		log_info( _("key %08lX: direct key signature added\n"),
-					 (ulong)keyid[1]);
+		if(!opt.quiet)
+		  log_info( _("key %08lX: direct key signature added\n"),
+			    (ulong)keyid[1]);
 	    }
 	}
     }
@@ -1680,20 +1699,12 @@ merge_sigs( KBNODE dst, KBNODE src, int *n_sigs,
 	    || n->pkt->pkt.signature->sig_class == 0x28 )
 	    continue; /* skip signatures which are only valid on subkeys */
 	found = 0;
-	for(n2=dst->next; n2 && n2->pkt->pkttype != PKT_USER_ID; n2 = n2->next){
-	    if( n2->pkt->pkttype == PKT_SIGNATURE
-		&& n->pkt->pkt.signature->keyid[0]
-		   == n2->pkt->pkt.signature->keyid[0]
-		&& n->pkt->pkt.signature->keyid[1]
-		   == n2->pkt->pkt.signature->keyid[1]
-		&& n->pkt->pkt.signature->timestamp
-		   <= n2->pkt->pkt.signature->timestamp
-		&& n->pkt->pkt.signature->sig_class
-		   == n2->pkt->pkt.signature->sig_class ) {
-		found++;
-		break;
+	for(n2=dst->next; n2 && n2->pkt->pkttype != PKT_USER_ID; n2 = n2->next)
+	  if(!cmp_signatures(n->pkt->pkt.signature,n2->pkt->pkt.signature))
+	    {
+	      found++;
+	      break;
 	    }
-	}
 	if( !found ) {
 	    /* This signature is new or newer, append N to DST.
 	     * We add a clone to the original keyblock, because this
