@@ -137,7 +137,12 @@ do_add_key_flags (PKT_signature *sig, unsigned int use)
 
     buf[0] = 0;
     if (use & PUBKEY_USAGE_SIG)
-        buf[0] |= 0x01 | 0x02;
+      {
+	if(sig->sig_class==0x18)
+	  buf[0] |= 0x02; /* Don't set the certify flag for subkeys */
+	else
+	  buf[0] |= 0x01 | 0x02;
+      }
     if (use & PUBKEY_USAGE_ENC)
         buf[0] |= 0x04 | 0x08;
     build_sig_subpkt (sig, SIGSUBPKT_KEY_FLAGS, buf, 1);
@@ -244,17 +249,17 @@ keygen_set_std_prefs (const char *string,int personal)
         string = "";
 
     for (s=string; *s; s = s2) {
-        if ((*s=='s' || *s == 'S') && isdigit(s[1]) ) {
+        if ((*s=='s' || *s == 'S') && digitp (s+1) ) {
             val = strtoul (++s, (char**)&s2, 10);
             if (set_one_pref (val, 'S', check_cipher_algo, sym, &nsym))
                 rc = -1;
         }
-        else if ((*s=='h' || *s == 'H') && isdigit(s[1]) ) {
+        else if ((*s=='h' || *s == 'H') && digitp (s+1) ) {
             val = strtoul (++s, (char**)&s2, 10);
             if (set_one_pref (val, 'H', check_digest_algo, hash, &nhash))
                 rc = -1;
         }
-        else if ((*s=='z' || *s == 'Z') && isdigit(s[1]) ) {
+        else if ((*s=='z' || *s == 'Z') && digitp (s+1) ) {
             val = strtoul (++s, (char**)&s2, 10);
             if (set_one_pref (val, 'Z', check_compress_algo, zip, &nzip))
                 rc = -1;
@@ -267,7 +272,7 @@ keygen_set_std_prefs (const char *string,int personal)
 	  mdc=0;
 	  s2=s+6;
 	}
-        else if (isspace (*s))
+        else if (spacep (s))
             s2 = s+1;
         else {
             log_info (_("invalid character in preference string\n"));
@@ -915,10 +920,10 @@ gen_rsa(int algo, unsigned nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
 static int
 check_valid_days( const char *s )
 {
-    if( !isdigit(*s) )
+    if( !digitp(s) )
 	return 0;
     for( s++; *s; s++)
-	if( !isdigit(*s) )
+	if( !digitp(s) )
 	    break;
     if( !*s )
 	return 1;
@@ -996,15 +1001,18 @@ ask_algo (int addmode, unsigned int *r_usage)
 				      _("Create anyway? ")))
 	      {
 		algo = PUBKEY_ALGO_ELGAMAL;
+		*r_usage = PUBKEY_USAGE_ENC | PUBKEY_USAGE_SIG;
 		break;
 	      }
 	}
 	else if( algo == 3 && addmode ) {
 	    algo = PUBKEY_ALGO_ELGAMAL_E;
+            *r_usage = PUBKEY_USAGE_ENC;
 	    break;
 	}
 	else if( algo == 2 ) {
 	    algo = PUBKEY_ALGO_DSA;
+            *r_usage = PUBKEY_USAGE_SIG;
 	    break;
 	}
 	else
@@ -1252,7 +1260,7 @@ ask_user_id( int mode )
 
 		if( strpbrk( aname, "<>" ) )
 		    tty_printf(_("Invalid character in name\n"));
-		else if( isdigit(*aname) )
+		else if( digitp(aname) )
 		    tty_printf(_("Name may not start with a digit\n"));
 		else if( strlen(aname) < 5 )
 		    tty_printf(_("Name must be at least 5 characters long\n"));
@@ -1266,7 +1274,7 @@ ask_user_id( int mode )
 		amail = cpr_get("keygen.email",_("Email address: "));
 		trim_spaces(amail);
 		cpr_kill_prompt();
-		if( !*amail )
+		if( !*amail || opt.allow_freeform_uid )
 		    break;   /* no email address is okay */
 		else if( has_invalid_email_chars(amail)
 			 || string_count_chr(amail,'@') != 1
@@ -1319,11 +1327,13 @@ ask_user_id( int mode )
 
 	tty_printf(_("You selected this USER-ID:\n    \"%s\"\n\n"), uid);
 	/* fixme: add a warning if this user-id already exists */
-	if( !*amail && (strchr( aname, '@' ) || strchr( acomment, '@'))) {
+	if( !*amail && !opt.allow_freeform_uid
+	    && (strchr( aname, '@' ) || strchr( acomment, '@')))
+	  {
 	    fail = 1;
 	    tty_printf(_("Please don't put the email address "
-			  "into the real name or the comment\n") );
-	}
+			 "into the real name or the comment\n") );
+	  }
 
 	for(;;) {
 	    const char *ansstr = _("NnCcEeOoQq");
@@ -1446,7 +1456,7 @@ do_create( int algo, unsigned int nbits, KBNODE pub_root, KBNODE sec_root,
     else
 	BUG();
 
-  #ifdef ENABLE_COMMENT_PACKETS
+#ifdef ENABLE_COMMENT_PACKETS
     if( !rc ) {
 	add_kbnode( pub_root,
 		make_comment_node("#created by GNUPG v" VERSION " ("
@@ -1455,7 +1465,7 @@ do_create( int algo, unsigned int nbits, KBNODE pub_root, KBNODE sec_root,
 		make_comment_node("#created by GNUPG v" VERSION " ("
 					    PRINTABLE_OS_NAME ")"));
     }
-  #endif
+#endif
     return rc;
 }
 
@@ -1522,7 +1532,7 @@ get_parameter_algo( struct para_data_s *para, enum para_name key )
     struct para_data_s *r = get_parameter( para, key );
     if( !r )
 	return -1;
-    if( isdigit( *r->u.value ) )
+    if( digitp( r->u.value ) )
 	i = atoi( r->u.value );
     else
         i = string_to_pubkey_algo( r->u.value );
@@ -2003,6 +2013,11 @@ generate_keypair( const char *fname )
 	strcpy( r->u.value, "1024" );
 	r->next = para;
 	para = r;
+	r = m_alloc_clear( sizeof *r + 20 );
+	r->key = pKEYUSAGE;
+	strcpy( r->u.value, "sign" );
+	r->next = para;
+	para = r;
 
 	algo = PUBKEY_ALGO_ELGAMAL_E;
 	r = m_alloc_clear( sizeof *r + 20 );
@@ -2010,6 +2025,11 @@ generate_keypair( const char *fname )
 	sprintf( r->u.value, "%d", algo );
 	r->next = para;
 	para = r;
+	r = m_alloc_clear( sizeof *r + 20 );
+	r->key = pSUBKEYUSAGE;
+	strcpy( r->u.value, "encrypt" );
+	r->next = para;
+	para = r;
     }
     else {
 	r = m_alloc_clear( sizeof *r + 20 );
