@@ -87,6 +87,16 @@ void deprecated_warning(const char *configname,unsigned int configlineno,
 			const char *option,const char *repl1,const char *repl2);
 const char *compress_algo_to_string(int algo);
 int check_compress_algo(int algo);
+const char *compliance_option_string(void);
+void compliance_failure(void);
+
+struct parse_options
+{
+  char *name;
+  unsigned int bit;
+};
+
+int parse_options(char *str,unsigned int *options,struct parse_options *opts);
 
 /*-- helptext.c --*/
 void display_online_help( const char *keyword );
@@ -199,6 +209,8 @@ void list_keyblock( KBNODE keyblock, int secret, int fpr, void *opaque );
 void print_fingerprint (PKT_public_key *pk, PKT_secret_key *sk, int mode);
 void show_policy_url(PKT_signature *sig,int indent);
 void show_notation(PKT_signature *sig,int indent);
+void dump_attribs(const PKT_user_id *uid,
+		  PKT_public_key *pk,PKT_secret_key *sk);
 void set_attrib_fd(int fd);
 
 /*-- verify.c --*/
