@@ -660,15 +660,12 @@ proc_compressed( CTX c, PACKET *pkt )
  * Returns: 0 = valid signature or an error code
  */
 static int
-do_check_sig( CTX c, KBNODE node, int *is_selfsig, int *is_expkey )
+do_check_sig( CTX c, KBNODE node, int *is_selfsig,
+	      int *is_expkey, int *is_revkey )
 {
     PKT_signature *sig;
     MD_HANDLE md = NULL, md2 = NULL;
-    int algo, rc, dum2;
-    u32 dummy;
-
-    if(!is_expkey)
-      is_expkey=&dum2;
+    int algo, rc;
 
     assert( node->pkt->pkttype == PKT_SIGNATURE );
     if( is_selfsig )
@@ -722,9 +719,9 @@ do_check_sig( CTX c, KBNODE node, int *is_selfsig, int *is_expkey )
     }
     else
 	return G10ERR_SIG_CLASS;
-    rc = signature_check2( sig, md, &dummy, is_expkey );
+    rc = signature_check2( sig, md, NULL, is_expkey, is_revkey );
     if( rc == G10ERR_BAD_SIGN && md2 )
-	rc = signature_check2( sig, md2, &dummy, is_expkey );
+	rc = signature_check2( sig, md2, NULL, is_expkey, is_revkey );
     md_close(md);
     md_close(md2);
 
@@ -993,7 +990,7 @@ list_node( CTX c, KBNODE node )
 	    fputs("sig", stdout);
 	if( opt.check_sigs ) {
 	    fflush(stdout);
-	    switch( (rc2=do_check_sig( c, node, &is_selfsig, NULL )) ) {
+	    switch( (rc2=do_check_sig( c, node, &is_selfsig, NULL, NULL )) ) {
 	      case 0:		       sigrc = '!'; break;
 	      case G10ERR_BAD_SIGN:    sigrc = '-'; break;
 	      case G10ERR_NO_PUBKEY: 
@@ -1242,7 +1239,7 @@ check_sig_and_print( CTX c, KBNODE node )
 {
     PKT_signature *sig = node->pkt->pkt.signature;
     const char *astr, *tstr;
-    int rc, is_expkey=0;
+    int rc, is_expkey=0, is_revkey=0;
 
     if( opt.skip_verify ) {
 	log_info(_("signature verification suppressed\n"));
@@ -1303,19 +1300,38 @@ check_sig_and_print( CTX c, KBNODE node )
             }
         }
     }
-    
-
 
     tstr = asctimestamp(sig->timestamp);
     astr = pubkey_algo_to_string( sig->pubkey_algo );
     log_info(_("Signature made %.*s using %s key ID %08lX\n"),
 	    (int)strlen(tstr), tstr, astr? astr: "?", (ulong)sig->keyid[1] );
 
-    rc = do_check_sig(c, node, NULL, &is_expkey );
+    rc = do_check_sig(c, node, NULL, &is_expkey, &is_revkey );
     if( rc == G10ERR_NO_PUBKEY && opt.keyserver_scheme && opt.keyserver_options.auto_key_retrieve) {
 	if( keyserver_import_keyid ( sig->keyid )==0 )
-	    rc = do_check_sig(c, node, NULL, &is_expkey );
+	    rc = do_check_sig(c, node, NULL, &is_expkey, &is_revkey );
     }
+
+    /* If the key still isn't found, try to inform the user where it
+       can be found. */
+    if(rc==G10ERR_NO_PUBKEY)
+      {
+	const byte *p;
+	int seq=0;
+	size_t n;
+
+	while((p=enum_sig_subpkt(sig->hashed,SIGSUBPKT_PREF_KS,&n,&seq,NULL)))
+	  {
+	    /* According to my favorite copy editor, in English
+	       grammar, you say "at" if the key is located on a web
+	       page, but "from" if it is located on a keyserver.  I'm
+	       not going to even try to make two strings here :) */
+	    log_info(_("Key available at: ") );
+	    print_string( log_stream(), p, n, 0 );
+	    putc( '\n', log_stream() );
+	  }
+      }
+
     if( !rc || rc == G10ERR_BAD_SIGN ) {
 	KBNODE un, keyblock;
 	int count=0, statno;
@@ -1327,6 +1343,8 @@ check_sig_and_print( CTX c, KBNODE node )
 	  statno=STATUS_EXPSIG;
 	else if(is_expkey)
 	  statno=STATUS_EXPKEYSIG;
+	else if(is_revkey)
+	  statno=STATUS_REVKEYSIG;
 	else
 	  statno=STATUS_GOODSIG;
 
@@ -1421,9 +1439,14 @@ check_sig_and_print( CTX c, KBNODE node )
 		     !un->pkt->pkt.user_id->attrib_data )
 		    continue;
 
-		if(opt.show_photos && un->pkt->pkt.user_id->attrib_data)
-		  show_photos(un->pkt->pkt.user_id->attribs,
-			      un->pkt->pkt.user_id->numattribs,pk,NULL);
+		if(un->pkt->pkt.user_id->attrib_data)
+		  {
+		    dump_attribs(un->pkt->pkt.user_id,pk,NULL);
+
+		    if(opt.show_photos)
+		      show_photos(un->pkt->pkt.user_id->attribs,
+				  un->pkt->pkt.user_id->numattribs,pk,NULL);
+		  }
 
 		log_info(    _("                aka \""));
                 print_utf8_string( log_stream(), un->pkt->pkt.user_id->name,
@@ -1612,10 +1635,30 @@ proc_tree( CTX c, KBNODE node )
     }
     else if( node->pkt->pkttype == PKT_SIGNATURE ) {
 	PKT_signature *sig = node->pkt->pkt.signature;
+	int multiple_ok=1;
 
-	if(find_next_kbnode(node, PKT_SIGNATURE))
-	  log_info(_("WARNING: multiple signatures detected.  "
-		     "Only the first will be checked.\n"));
+	n1=find_next_kbnode(node, PKT_SIGNATURE);
+	if(n1)
+	  {
+	    byte class=sig->sig_class;
+	    byte hash=sig->digest_algo;
+
+	    for(; n1; (n1 = find_next_kbnode(n1, PKT_SIGNATURE)))
+	      {
+		/* We can't currently handle multiple signatures of
+		   different classes or digests (we'd pretty much have
+		   to run a different hash context for each), but if
+		   they are all the same, make an exception. */
+		if(n1->pkt->pkt.signature->sig_class!=class
+		   || n1->pkt->pkt.signature->digest_algo!=hash)
+		  {
+		    multiple_ok=0;
+		    log_info(_("WARNING: multiple signatures detected.  "
+			       "Only the first will be checked.\n"));
+		    break;
+		  }
+	      }
+	  }
 
 	if( sig->sig_class != 0x00 && sig->sig_class != 0x01 )
 	    log_info(_("standalone signature of class 0x%02x\n"),
@@ -1638,7 +1681,7 @@ proc_tree( CTX c, KBNODE node )
 		 * signature has been created in textmode */
 		c->mfx.md2 = md_open( sig->digest_algo, 0 );
 	    }
-	  #if 0 /* workaround disabled */
+#if 0 /* workaround disabled */
 	    /* Here we have another hack to work around a pgp 2 bug
 	     * It works by not using the textmode for detached signatures;
 	     * this will let the first signature check (on md) fail
@@ -1647,7 +1690,7 @@ proc_tree( CTX c, KBNODE node )
 	     * hack but it may help in some cases (and break others)
 	     */
 		    /*	c->mfx.md2? 0 :(sig->sig_class == 0x01) */
-	  #endif
+#endif
             if ( DBG_HASHING ) {
                 md_start_debug( c->mfx.md, "verify" );
                 if ( c->mfx.md2  )
@@ -1677,7 +1720,11 @@ proc_tree( CTX c, KBNODE node )
 	else if (!opt.quiet)
 	    log_info(_("old style (PGP 2.x) signature\n"));
 
-	check_sig_and_print( c, node );
+	if(multiple_ok)
+	  for( n1 = node; n1; (n1 = find_next_kbnode(n1, PKT_SIGNATURE )) )
+	    check_sig_and_print( c, n1 );
+	else
+	  check_sig_and_print( c, node );
     }
     else {
         dump_kbnode (c->list);
@@ -1685,7 +1732,3 @@ proc_tree( CTX c, KBNODE node )
         dump_kbnode (node);
     }
 }
-
-
-
-
