@@ -1,5 +1,6 @@
 /* options.h
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002,
+ *               2003  Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -28,11 +29,11 @@
 
 #ifndef EXTERN_UNLESS_MAIN_MODULE
 /* Norcraft can't cope with common symbols */
- #if defined (__riscos__) && !defined (INCLUDED_BY_MAIN_MODULE)
-  #define EXTERN_UNLESS_MAIN_MODULE extern
- #else
-  #define EXTERN_UNLESS_MAIN_MODULE 
- #endif
+#if defined (__riscos__) && !defined (INCLUDED_BY_MAIN_MODULE)
+#define EXTERN_UNLESS_MAIN_MODULE extern
+#else
+#define EXTERN_UNLESS_MAIN_MODULE 
+#endif
 #endif
 
 EXTERN_UNLESS_MAIN_MODULE
@@ -89,12 +90,10 @@ struct {
     int compress_keys;
     int compress_sigs;
     int always_trust;
-    int pgp2;
-    int pgp6;
-    int pgp7; /* if we get any more of these, it's time to look at a
-		 special emulate_pgp variable... */
-    int rfc1991;
-    int rfc2440;
+    enum
+      {
+	CO_GNUPG=0, CO_RFC2440, CO_RFC1991, CO_PGP2, CO_PGP6, CO_PGP7, CO_PGP8
+      } compliance;
     int pgp2_workarounds;
     unsigned int emulate_bugs; /* bug emulation flags EMUBUG_xxxx */
     int shm_coprocess;
@@ -177,6 +176,8 @@ struct {
     int no_homedir_creation;
     int show_keyring;
     struct groupitem *grouplist;
+    int mangle_dos_filenames;
+    int enable_progress_filter;
 } opt;
 
 
@@ -203,5 +204,12 @@ struct {
 #define DBG_HASHING (opt.debug & DBG_HASHING_VALUE)
 #define DBG_EXTPROG (opt.debug & DBG_EXTPROG_VALUE)
 
+#define GNUPG   (opt.compliance==CO_GNUPG)
+#define RFC1991 (opt.compliance==CO_RFC1991 || opt.compliance==CO_PGP2)
+#define RFC2440 (opt.compliance==CO_RFC2440)
+#define PGP2    (opt.compliance==CO_PGP2)
+#define PGP6    (opt.compliance==CO_PGP6)
+#define PGP7    (opt.compliance==CO_PGP7)
+#define PGP8    (opt.compliance==CO_PGP8)
 
 #endif /*G10_OPTIONS_H*/
