@@ -1,5 +1,6 @@
 /* parse-packet.c  - read packets
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002,
+ *               2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -41,9 +42,9 @@ static int list_mode = 0;
 
 static int  parse( IOBUF inp, PACKET *pkt, int onlykeypkts,
                   off_t *retpos, int *skip, IOBUF out, int do_skip
-	    #ifdef DEBUG_PARSE_PACKET
+#ifdef DEBUG_PARSE_PACKET
 		   ,const char *dbg_w, const char *dbg_f, int dbg_l
-	    #endif
+#endif
 		 );
 static int  copy_packet( IOBUF inp, IOBUF out, int pkttype,
 					       unsigned long pktlen );
@@ -366,8 +367,28 @@ parse( IOBUF inp, PACKET *pkt, int onlykeypkts, off_t *retpos,
 	lenbytes = ((ctb&3)==3)? 0 : (1<<(ctb & 3));
 	if( !lenbytes ) {
 	    pktlen = 0; /* don't know the value */
-	    if( pkttype != PKT_COMPRESSED )
-		iobuf_set_block_mode(inp, 1);
+            switch (pkttype) {
+              case PKT_ENCRYPTED:
+              case PKT_PLAINTEXT:
+                /* These partial length encodings are from an very
+		   early GnuPG release and deprecated.  However we
+		   still support them read-wise.  Note, that we should
+		   not allow them for any key related packets, because
+		   this might render a keyring unusable if an errenous
+		   packet indicated this mode but not complying to it
+		   gets imported. */
+                iobuf_set_block_mode(inp, 1);
+		break;
+
+            case PKT_COMPRESSED:
+              break; /* the orginal pgp 2 way. */
+
+            default:
+              log_error ("%s: old style partial length "
+                         "for invalid packet type\n", iobuf_where(inp) );
+              rc = G10ERR_INVALID_PACKET;
+              goto leave;
+            }
 	}
 	else {
 	    for( ; lenbytes; lenbytes-- ) {
@@ -752,12 +773,12 @@ dump_sig_subpkt( int hashed, int type, int critical,
                type, (unsigned)length );
     }
     
+    buffer++;
+    length--;
    
     printf("\t%s%ssubpkt %d len %u (", /*)*/
 	      critical ? "critical ":"",
 	      hashed ? "hashed ":"", type, (unsigned)length );
-    buffer++;
-    length--;
     if( length > buflen ) {
 	printf("too short: buffer is only %u)\n", (unsigned)buflen );
 	return;
@@ -823,8 +844,6 @@ dump_sig_subpkt( int hashed, int type, int critical,
 	    fputs("notation: ", stdout );
 	    if( length < 8 )
 		p = "[too short]";
-	    else if( !(*buffer & 0x80) )
-		p = "[not human readable]";
 	    else {
 		const byte *s = buffer;
 		size_t n1, n2;
@@ -837,7 +856,11 @@ dump_sig_subpkt( int hashed, int type, int critical,
 		else {
 		    print_string( stdout, s, n1, ')' );
 		    putc( '=', stdout );
-		    print_string( stdout, s+n1, n2, ')' );
+
+		    if( *buffer & 0x80 )
+		      print_string( stdout, s+n1, n2, ')' );
+		    else
+		      p = "[not human readable]";
 		}
 	    }
 	}
@@ -858,7 +881,8 @@ dump_sig_subpkt( int hashed, int type, int critical,
 	  printf(" %02X", buffer[i]);
 	break;
       case SIGSUBPKT_PREF_KS:
-	p = "preferred key server";
+	fputs("preferred key server: ", stdout );
+	print_string( stdout, buffer, length, ')' );
 	break;
       case SIGSUBPKT_PRIMARY_UID:
 	p = "primary user ID";
@@ -897,10 +921,12 @@ dump_sig_subpkt( int hashed, int type, int critical,
         for( i=0; i < length; i++ )
             printf(" %02x", buffer[i] );
 	break;
-      case SIGSUBPKT_PRIV_VERIFY_CACHE:
-	p = "obsolete verification cache";
+      default:
+	if(type>=100 && type<=110)
+	  p="experimental / private subpacket";
+	else
+	  p = "?";
 	break;
-      default: p = "?"; break;
     }
 
     printf("%s)\n", p? p: "");
@@ -932,6 +958,7 @@ parse_one_sig_subpkt( const byte *buffer, size_t n, int type )
       case SIGSUBPKT_PREF_HASH:
       case SIGSUBPKT_PREF_COMPR:
       case SIGSUBPKT_POLICY:
+      case SIGSUBPKT_PREF_KS:
       case SIGSUBPKT_FEATURES:
 	return 0;
       case SIGSUBPKT_EXPORTABLE:
@@ -955,19 +982,6 @@ parse_one_sig_subpkt( const byte *buffer, size_t n, int type )
           if ( n != 1 )
               break;
           return 0;   
-      case SIGSUBPKT_PRIV_VERIFY_CACHE:
-        /* We used this in gpg 1.0.5 and 1.0.6 to cache signature
-         * verification results - it is no longer used.
-         * "GPG" 0x00 <mode> <stat>
-         * where mode == 1: valid data, stat == 0: invalid signature
-         * stat == 1: valid signature 
-	 * (because we use private data, we check our marker) */
-	if( n < 6 )
-	    break;
-	if( buffer[0] != 'G' || buffer[1] != 'P'
-            || buffer[2] != 'G' || buffer[3] )
-	    return -2;
-	return 4;
       default: return -1;
     }
     return -3;
@@ -996,7 +1010,9 @@ can_handle_critical( const byte *buffer, size_t n, int type )
       case SIGSUBPKT_KEY_FLAGS:
       case SIGSUBPKT_PRIMARY_UID:
       case SIGSUBPKT_FEATURES:
-      case SIGSUBPKT_POLICY: /* Is it enough to show the policy? */
+	/* Is it enough to show the policy or keyserver? */
+      case SIGSUBPKT_POLICY:
+      case SIGSUBPKT_PREF_KS:
 	return 1;
 
       default:
@@ -1061,13 +1077,15 @@ enum_sig_subpkt( const subpktarea_t *pktbuf, sigsubpkttype_t reqtype,
 	    if( *critical ) {
 		if( n-1 > buflen+1 )
 		    goto too_short;
-		if( !can_handle_critical(buffer+1, n-1, type ) ) {
-		    log_info(_("subpacket of type %d has critical bit set\n"),
-									type);
+		if( !can_handle_critical(buffer+1, n-1, type ) )
+		  {
+		    if(opt.verbose)
+		      log_info(_("subpacket of type %d has "
+				 "critical bit set\n"),type);
 		    if( start )
-			*start = seq;
+		      *start = seq;
 		    return NULL; /* this is an error */
-		}
+		  }
 	    }
 	}
 	else if( reqtype < 0 ) /* list packets */
@@ -1220,11 +1238,8 @@ parse_signature( IOBUF inp, int pkttype, unsigned long pktlen,
 	    goto leave;
 	}
 	if( n ) {
-            /* we add 8 extra bytes so that we have space for the signature
-             * status cache.  Well we are wastin this if there is a cache
-             * packet already, but in the other case it avoids an realloc */
-	    sig->unhashed = m_alloc (sizeof(*sig->unhashed) + n + 8 - 1 );
-            sig->unhashed->size = n + 8;
+	    sig->unhashed = m_alloc (sizeof(*sig->unhashed) + n - 1 );
+            sig->unhashed->size = n;
 	    sig->unhashed->len = n;
 	    if( iobuf_read(inp, sig->unhashed->data, n ) != n ) {
 		log_error("premature eof while reading "
@@ -1258,17 +1273,19 @@ parse_signature( IOBUF inp, int pkttype, unsigned long pktlen,
 	}
 
 	p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_SIG_CREATED, NULL );
-	if( !p )
-	    log_error("signature packet without timestamp\n");
-	else
-	    sig->timestamp = buffer_to_u32(p);
+	if(p)
+	  sig->timestamp = buffer_to_u32(p);
+	else if(!(sig->pubkey_algo>=100 && sig->pubkey_algo<=110))
+	  log_error("signature packet without timestamp\n");
+
 	p = parse_sig_subpkt2( sig, SIGSUBPKT_ISSUER, NULL );
-	if( !p )
-	    log_error("signature packet without keyid\n");
-	else {
+	if(p)
+	  {
 	    sig->keyid[0] = buffer_to_u32(p);
 	    sig->keyid[1] = buffer_to_u32(p+4);
-	}
+	  }
+	else if(!(sig->pubkey_algo>=100 && sig->pubkey_algo<=110))
+	  log_error("signature packet without keyid\n");
 
 	p=parse_sig_subpkt(sig->hashed,SIGSUBPKT_SIG_EXPIRE,NULL);
 	if(p)
@@ -1528,6 +1545,7 @@ parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 	pk->req_usage = 0; 
 	pk->pubkey_usage = 0; /* not yet used */
         pk->is_revoked = 0;
+	pk->is_disabled = 0;
 	pk->keyid[0] = 0;
 	pk->keyid[1] = 0;
     }
@@ -1543,6 +1561,7 @@ parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
     if( pkttype == PKT_SECRET_KEY || pkttype == PKT_SECRET_SUBKEY ) {
 	PKT_secret_key *sk = pkt->pkt.secret_key;
 	byte temp[16];
+        size_t snlen = 0;
 
 	if( !npkey ) {
 	    sk->skey[0] = mpi_set_opaque( NULL,
@@ -1615,6 +1634,8 @@ parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 		    break;
 		  case 1001: if( list_mode ) printf(  "\tgnu-dummy S2K" );
 		    break;
+		  case 1002: if (list_mode) printf("\tgnu-divert-to-card S2K");
+		    break;
 		  default:
 		    if( list_mode )
 			printf(  "\tunknown %sS2K %d\n",
@@ -1650,6 +1671,19 @@ parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 			printf("\tprotect count: %lu\n",
 					    (ulong)sk->protect.s2k.count);
 		}
+		else if( sk->protect.s2k.mode == 1002 ) {
+                    /* Read the serial number. */
+                    if (pktlen < 1) {
+                      rc = G10ERR_INVALID_PACKET;
+			goto leave;
+		    }
+		    snlen = iobuf_get (inp);
+		    pktlen--;
+                    if (pktlen < snlen || snlen == -1) {
+			rc = G10ERR_INVALID_PACKET;
+			goto leave;
+                    }
+		}
 	    }
 	    /* Note that a sk->protect.algo > 110 is illegal, but I'm
 	       not erroring on it here as otherwise there would be no
@@ -1679,6 +1713,8 @@ parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 	    }
 	    if( sk->protect.s2k.mode == 1001 )
 		sk->protect.ivlen = 0;
+	    else if( sk->protect.s2k.mode == 1002 )
+		sk->protect.ivlen = snlen < 16? snlen : 16;
 
 	    if( pktlen < sk->protect.ivlen ) {
 		rc = G10ERR_INVALID_PACKET;
@@ -1687,7 +1723,8 @@ parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 	    for(i=0; i < sk->protect.ivlen && pktlen; i++, pktlen-- )
 		temp[i] = iobuf_get_noeof(inp);
 	    if( list_mode ) {
-		printf(  "\tprotect IV: ");
+		printf( sk->protect.s2k.mode == 1002? "\tserial-number: "
+                                                    : "\tprotect IV: ");
 		for(i=0; i < sk->protect.ivlen; i++ )
 		    printf(" %02x", temp[i] );
 		putchar('\n');
@@ -1700,7 +1737,8 @@ parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 	 * If the user is so careless, not to protect his secret key,
 	 * we can assume, that he operates an open system :=(.
 	 * So we put the key into secure memory when we unprotect it. */
-	if( sk->protect.s2k.mode == 1001 ) {
+	if( sk->protect.s2k.mode == 1001 
+            || sk->protect.s2k.mode == 1002 ) {
 	    /* better set some dummy stuff here */
 	    sk->skey[npkey] = mpi_set_opaque(NULL, m_strdup("dummydata"), 10);
 	    pktlen = 0;
