@@ -1088,46 +1088,60 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned use )
    intersection"), and PGP has no mechanism to fix such a broken
    preference list, so I'm including it. -dms */
 
-static int
-algo_available( int preftype, int algo, void *hint )
+int
+algo_available( preftype_t preftype, int algo, void *hint )
 {
-    if( preftype == PREFTYPE_SYM ) {
-        if( opt.pgp6 && ( algo != 1 && algo != 2 && algo != 3) )
-	  return 0;
-
-        if( (opt.pgp7 || opt.pgp8)
-	    && (algo != 1 && algo != 2 && algo != 3
-		&& algo != 7 && algo != 8 && algo != 9 && algo != 10) )
-	  return 0;
-
-	return algo && !check_cipher_algo( algo );
-    }
-    else if( preftype == PREFTYPE_HASH ) {
-        int bits=0;
-
-	if(hint)
-	  bits=*(int *)hint;
-
-	if(bits && (bits != md_digest_length(algo)))
-	  return 0;
-
-        if( (opt.pgp6 || opt.pgp7) && (algo != 1 && algo != 2 && algo != 3) )
-	  return 0;
-
-	if( opt.pgp8 && (algo != 1 && algo != 2 && algo != 3 && algo != 8))
-	  return 0;
-
-	return algo && !check_digest_algo( algo );
-    }
-    else if( preftype == PREFTYPE_ZIP ) {
-        if ( ( opt.pgp6 || opt.pgp7 || opt.pgp8 )
-	     && ( algo !=0 && algo != 1) )
-	  return 0;
-
-	return !check_compress_algo( algo );
-    }
-    else
+  if( preftype == PREFTYPE_SYM )
+    {
+      if(PGP6 && (algo != CIPHER_ALGO_IDEA
+		  && algo != CIPHER_ALGO_3DES
+		  && algo != CIPHER_ALGO_CAST5))
 	return 0;
+      
+      if((PGP7 || PGP8) && (algo != CIPHER_ALGO_IDEA
+			    && algo != CIPHER_ALGO_3DES
+			    && algo != CIPHER_ALGO_CAST5
+			    && algo != CIPHER_ALGO_AES
+			    && algo != CIPHER_ALGO_AES192
+			    && algo != CIPHER_ALGO_AES256
+			    && algo != CIPHER_ALGO_TWOFISH))
+	return 0;
+
+      return algo && !check_cipher_algo( algo );
+    }
+  else if( preftype == PREFTYPE_HASH )
+    {
+      if(hint && ((*(int *)hint) != md_digest_length(algo)))
+	return 0;
+
+      if((PGP6 || PGP7) && (algo != DIGEST_ALGO_MD5
+			    && algo != DIGEST_ALGO_SHA1
+			    && algo != DIGEST_ALGO_RMD160))
+	return 0;
+
+
+      if(PGP8 && (algo != DIGEST_ALGO_MD5
+		  && algo != DIGEST_ALGO_SHA1
+		  && algo != DIGEST_ALGO_RMD160
+		  && algo != DIGEST_ALGO_SHA256))
+	return 0;
+
+      /* TIGER is not allowed any longer according to 2440bis. */
+      if( RFC2440 && algo == DIGEST_ALGO_TIGER )
+	return 0;
+
+      return algo && !check_digest_algo( algo );
+    }
+  else if( preftype == PREFTYPE_ZIP )
+    {
+      if((PGP6 || PGP7 || PGP8) && (algo != COMPRESS_ALGO_NONE
+				    && algo != COMPRESS_ALGO_ZIP))
+	return 0;
+
+      return !check_compress_algo( algo );
+    }
+  else
+    return 0;
 }
 
 
@@ -1154,7 +1168,7 @@ select_algo_from_prefs(PK_LIST pk_list, int preftype, int request, void *hint)
 
 	memset( mask, 0, 8 * sizeof *mask );
 	if( preftype == PREFTYPE_SYM ) {
-	  if( opt.pgp2 &&
+	  if( PGP2 &&
 	      pkr->pk->version < 4 &&
 	      pkr->pk->selfsigversion < 4 )
 	    mask[0] |= (1<<1); /* IDEA is implicitly there for v3 keys
@@ -1172,7 +1186,7 @@ select_algo_from_prefs(PK_LIST pk_list, int preftype, int request, void *hint)
 	     wasn't locked at MD5, we don't support sign+encrypt in
 	     --pgp2 mode, and that's the only time PREFTYPE_HASH is
 	     used anyway. -dms */
-	  if( opt.pgp2 &&
+	  if( PGP2 &&
 	      pkr->pk->version < 4 &&
 	      pkr->pk->selfsigversion < 4 )
 	    mask[0] |= (1<<1); /* MD5 is there for v3 keys with v3
@@ -1203,18 +1217,18 @@ select_algo_from_prefs(PK_LIST pk_list, int preftype, int request, void *hint)
 	    compr_hack = 1;
 	}
 
-      #if 0
+#if 0
 	log_debug("pref mask=%08lX%08lX%08lX%08lX%08lX%08lX%08lX%08lX\n",
 	       (ulong)mask[7], (ulong)mask[6], (ulong)mask[5], (ulong)mask[4],
 	     (ulong)mask[3], (ulong)mask[2], (ulong)mask[1], (ulong)mask[0]);
-      #endif
+#endif
 	for(i=0; i < 8; i++ )
 	    bits[i] &= mask[i];
-      #if 0
+#if 0
 	log_debug("pref bits=%08lX%08lX%08lX%08lX%08lX%08lX%08lX%08lX\n",
 	       (ulong)bits[7], (ulong)bits[6], (ulong)bits[5], (ulong)bits[4],
 	     (ulong)bits[3], (ulong)bits[2], (ulong)bits[1], (ulong)bits[0]);
-      #endif
+#endif
     }
     /* usable algorithms are now in bits
      * We now use the last key from pk_list to select
@@ -1261,9 +1275,9 @@ select_algo_from_prefs(PK_LIST pk_list, int preftype, int request, void *hint)
 	    }
     }
 
-  #if 0
+#if 0
     log_debug("prefs of type %d: selected %d\n", preftype, i );
-  #endif
+#endif
     if( compr_hack && !i ) {
 	/* selected no compression, but we should check whether
 	 * algorithm 1 is also available (the ordering is not relevant
