@@ -549,9 +549,11 @@ gen_revoke( const char *uname )
 	goto leave;
     }
 
-    if(opt.pgp2 || opt.pgp6 || opt.pgp7)
+    if(PGP2 || PGP6 || PGP7 || PGP8)
       {
-	rc=export_minimal_pk(out,NULL /*pub_keyblock*/,sig,NULL);
+	/* Use a minimal pk for PGPx mode, since PGP can't import bare
+	   revocation certificates. */
+	rc=export_minimal_pk(out,pub_keyblock,sig,NULL);
 	if(rc)
 	  goto leave;
       }
@@ -606,6 +608,7 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
     const char *code_text = NULL;
 
     do {
+        code=-1;
 	m_free(description);
 	description = NULL;
 
@@ -633,7 +636,7 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
 	      return NULL; /* cancel */
 	    if( hint && !*answer )
 		n = hint;
-	    else if(!isdigit( *answer ) )
+	    else if(!digitp( answer ) )
  	        n = -1;
 	    else
 		n = atoi(answer);
