@@ -1,5 +1,6 @@
 /* sig-check.c -  Check a signature
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003
+ *               Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -40,7 +41,7 @@ struct cmp_help_context_s {
 };
 
 static int do_check( PKT_public_key *pk, PKT_signature *sig,
-					 MD_HANDLE digest, int *r_expired );
+		     MD_HANDLE digest, int *r_expired, int *r_revoked );
 
 /****************
  * Check the signature which is contained in SIG.
@@ -50,20 +51,16 @@ static int do_check( PKT_public_key *pk, PKT_signature *sig,
 int
 signature_check( PKT_signature *sig, MD_HANDLE digest )
 {
-    u32 dummy;
-    int dum2;
-    return signature_check2( sig, digest, &dummy, &dum2 );
+    return signature_check2( sig, digest, NULL, NULL, NULL );
 }
 
 int
 signature_check2( PKT_signature *sig, MD_HANDLE digest,
-		  u32 *r_expiredate, int *r_expired )
+		  u32 *r_expiredate, int *r_expired, int *r_revoked )
 {
     PKT_public_key *pk = m_alloc_clear( sizeof *pk );
     int rc=0;
 
-    *r_expiredate = 0;
-
     /* Sanity check that the md has a context for the hash that the
        sig is expecting.  This can happen if a onepass sig header does
        not match the actual sig, and also if the clearsign "Hash:"
@@ -79,8 +76,9 @@ signature_check2( PKT_signature *sig, MD_HANDLE digest,
         rc=G10ERR_BAD_PUBKEY; /* you cannot have a good sig from an
 				 invalid subkey */
     else {
-	*r_expiredate = pk->expiredate;
-	rc = do_check( pk, sig, digest, r_expired );
+        if(r_expiredate)
+	  *r_expiredate = pk->expiredate;
+	rc = do_check( pk, sig, digest, r_expired, r_revoked );
     }
 
     free_public_key( pk );
@@ -135,7 +133,7 @@ signature_check2( PKT_signature *sig, MD_HANDLE digest,
 static int
 cmp_help( void *opaque, MPI result )
 {
-  #if 0 /* we do not use this anymore */
+#if 0 /* we do not use this anymore */
     int rc=0, i, j, c, old_enc;
     byte *dp;
     const byte *asn;
@@ -198,17 +196,21 @@ cmp_help( void *opaque, MPI result )
 	    return G10ERR_BAD_SIGN;
     }
     return 0;
-  #else
+#else
     return -1;
-  #endif
+#endif
 }
 
 static int
-do_check_messages( PKT_public_key *pk, PKT_signature *sig, int *r_expired )
+do_check_messages( PKT_public_key *pk, PKT_signature *sig,
+		   int *r_expired, int *r_revoked )
 {
     u32 cur_time;
 
-    *r_expired = 0;
+    if(r_expired)
+      *r_expired = 0;
+    if(r_revoked)
+      *r_revoked = 0;
     if( pk->version == 4 && pk->pubkey_algo == PUBKEY_ALGO_ELGAMAL_E ) {
 	log_info(_("key %08lX: this is a PGP generated "
 		   "ElGamal key which is NOT secure for signatures!\n"),
@@ -251,22 +253,26 @@ do_check_messages( PKT_public_key *pk, PKT_signature *sig, int *r_expired )
 	sprintf(buf,"%lu",(ulong)pk->expiredate);
 	write_status_text(STATUS_KEYEXPIRED,buf);
 	write_status(STATUS_SIGEXPIRED);
-	*r_expired = 1;
+	if(r_expired)
+	  *r_expired = 1;
     }
 
+    if(pk->is_revoked && r_revoked)
+      *r_revoked=1;
+
     return 0;
 }
 
 
 static int
 do_check( PKT_public_key *pk, PKT_signature *sig, MD_HANDLE digest,
-						    int *r_expired )
+	  int *r_expired, int *r_revoked )
 {
     MPI result = NULL;
     int rc=0;
     struct cmp_help_context_s ctx;
 
-    if( (rc=do_check_messages(pk,sig,r_expired)) )
+    if( (rc=do_check_messages(pk,sig,r_expired,r_revoked)) )
         return rc;
     if( (rc=check_digest_algo(sig->digest_algo)) )
 	return rc;
@@ -473,13 +479,13 @@ check_revocation_keys(PKT_public_key *pk,PKT_signature *sig)
 int
 check_key_signature( KBNODE root, KBNODE node, int *is_selfsig )
 {
-    u32 dummy;
-    int dum2;
-    return check_key_signature2(root, node, NULL, is_selfsig, &dummy, &dum2 );
+    return check_key_signature2(root, node, NULL, is_selfsig, NULL, NULL );
 }
 
 /* If check_pk is set, then use it to check the signature in node
    rather than getting it from root or the keydb. */
+/* TODO: add r_revoked here as well.  It has the same problems as
+   r_expiredate and r_expired and the cache. */
 int
 check_key_signature2( KBNODE root, KBNODE node, PKT_public_key *check_pk,
 		      int *is_selfsig, u32 *r_expiredate, int *r_expired )
@@ -492,8 +498,10 @@ check_key_signature2( KBNODE root, KBNODE node, PKT_public_key *check_pk,
 
     if( is_selfsig )
 	*is_selfsig = 0;
-    *r_expiredate = 0;
-    *r_expired = 0;
+    if( r_expiredate )
+        *r_expiredate = 0;
+    if( r_expired )
+        *r_expired = 0;
     assert( node->pkt->pkttype == PKT_SIGNATURE );
     assert( root->pkt->pkttype == PKT_PUBLIC_KEY );
 
@@ -511,7 +519,9 @@ check_key_signature2( KBNODE root, KBNODE node, PKT_public_key *check_pk,
 		if( keyid[0] == sig->keyid[0] && keyid[1] == sig->keyid[1] )
 		    *is_selfsig = 1;
 	    }
-	    if((rc=do_check_messages(pk,sig,r_expired)))
+	    /* BUG: This is wrong for non-self-sigs.. needs to be the
+	       actual pk */
+	    if((rc=do_check_messages(pk,sig,r_expired,NULL)))
 	      return rc;
             return sig->flags.valid? 0 : G10ERR_BAD_SIGN;
         }
@@ -531,7 +541,7 @@ check_key_signature2( KBNODE root, KBNODE node, PKT_public_key *check_pk,
 	  {
 	    md = md_open( algo, 0 );
 	    hash_public_key( md, pk );
-	    rc = do_check( pk, sig, md, r_expired );
+	    rc = do_check( pk, sig, md, r_expired, NULL );
 	    cache_sig_result ( sig, rc );
 	    md_close(md);
 	  }
@@ -543,12 +553,12 @@ check_key_signature2( KBNODE root, KBNODE node, PKT_public_key *check_pk,
 	    md = md_open( algo, 0 );
 	    hash_public_key( md, pk );
 	    hash_public_key( md, snode->pkt->pkt.public_key );
-	    rc = do_check( pk, sig, md, r_expired );
+	    rc = do_check( pk, sig, md, r_expired, NULL );
             cache_sig_result ( sig, rc );
 	    md_close(md);
 	}
 	else {
-            if (!opt.quiet)
+            if (opt.verbose)
                 log_info (_("key %08lX: no subkey for subkey "
 			    "revocation packet\n"),
                           (ulong)keyid_from_pk (pk, NULL));
@@ -569,21 +579,22 @@ check_key_signature2( KBNODE root, KBNODE node, PKT_public_key *check_pk,
 	    md = md_open( algo, 0 );
 	    hash_public_key( md, pk );
 	    hash_public_key( md, snode->pkt->pkt.public_key );
-	    rc = do_check( pk, sig, md, r_expired );
+	    rc = do_check( pk, sig, md, r_expired, NULL );
             cache_sig_result ( sig, rc );
 	    md_close(md);
 	}
 	else {
             if (opt.verbose)
-                log_info ("key %08lX: no subkey for subkey binding packet\n",
-                          (ulong)keyid_from_pk (pk, NULL));
+                log_info(_("key %08lX: no subkey for subkey "
+			   "binding signature\n"),
+			 (ulong)keyid_from_pk (pk, NULL));
 	    rc = G10ERR_SIG_CLASS;
 	}
     }
     else if( sig->sig_class == 0x1f ) { /* direct key signature */
 	md = md_open( algo, 0 );
 	hash_public_key( md, pk );
-	rc = do_check( pk, sig, md, r_expired );
+	rc = do_check( pk, sig, md, r_expired, NULL );
         cache_sig_result ( sig, rc );
 	md_close(md);
     }
@@ -601,12 +612,12 @@ check_key_signature2( KBNODE root, KBNODE node, PKT_public_key *check_pk,
 	      {
 		if( is_selfsig )
 		  *is_selfsig = 1;
-		rc = do_check( pk, sig, md, r_expired );
+		rc = do_check( pk, sig, md, r_expired, NULL );
 	      }
 	    else if (check_pk)
-	      rc=do_check(check_pk,sig,md,r_expired);
+	      rc=do_check(check_pk,sig,md,r_expired,NULL);
 	    else
-	      rc = signature_check2( sig, md, r_expiredate, r_expired );
+	      rc = signature_check2( sig, md, r_expiredate, r_expired, NULL );
 
             cache_sig_result ( sig, rc );
 	    md_close(md);
