@@ -332,7 +332,9 @@ tdbio_sync()
     return 0;
 }
 
-
+#if 0
+/* The transaction code is disabled in the 1.2.x branch, as it is not
+   yet used.  It will be enabled in 1.3.x. */
 
 /****************
  * Simple transactions system:
@@ -402,7 +404,7 @@ tdbio_cancel_transaction()
     in_transaction = 0;
     return 0;
 }
-
+#endif
 
 
 /********************************************************
@@ -427,9 +429,10 @@ create_version_record (void)
   memset( &rec, 0, sizeof rec );
   rec.r.ver.version = 3;
   rec.r.ver.created = make_timestamp();
-  rec.r.ver.marginals =  opt.marginals_needed;
-  rec.r.ver.completes =  opt.completes_needed;
+  rec.r.ver.marginals  = opt.marginals_needed;
+  rec.r.ver.completes  = opt.completes_needed;
   rec.r.ver.cert_depth = opt.max_cert_depth;
+  rec.r.ver.trust_model= 0;
   rec.rectype = RECTYPE_VER;
   rec.recnum = 0;
   rc = tdbio_write_record( &rec );
@@ -450,9 +453,18 @@ tdbio_set_dbname( const char *new_dbname, int create )
 	atexit( cleanup );
 	initialized = 1;
     }
-    fname = new_dbname? m_strdup( new_dbname )
-		      : make_filename(opt.homedir,
-		                      "trustdb" EXTSEP_S "gpg", NULL );
+
+    if(new_dbname==NULL)
+      fname=make_filename(opt.homedir,"trustdb" EXTSEP_S "gpg", NULL);
+    else if (*new_dbname != DIRSEP_C )
+      {
+	if (strchr(new_dbname, DIRSEP_C) )
+	  fname = make_filename (new_dbname, NULL);
+	else
+	  fname = make_filename (opt.homedir, new_dbname, NULL);
+      }
+    else
+      fname = m_strdup (new_dbname);
 
     if( access( fname, R_OK ) ) {
 	if( errno != ENOENT ) {
@@ -676,10 +688,16 @@ tdbio_write_nextcheck (ulong stamp)
 	log_fatal( _("%s: error reading version record: %s\n"),
 				       db_name, g10_errstr(rc) );
 
-    if (vr.r.ver.nextcheck == stamp)
+    if (vr.r.ver.nextcheck == stamp && vr.r.ver.trust_model == 0)
       return 0;
 
     vr.r.ver.nextcheck = stamp;
+    /* Force this to the classic trustdb since that is what we are
+       building.  This is to help smooth the 1.2->1.4 switchover, as
+       otherwise a 1.4 trustdb could be rebuilt as classic on 1.2
+       while still claiming to be a non-classic trust model. */
+    vr.r.ver.trust_model = 0;
+
     rc = tdbio_write_record( &vr );
     if( rc )
 	log_fatal( _("%s: error writing version record: %s\n"),
@@ -1158,7 +1176,8 @@ tdbio_read_record( ulong recnum, TRUSTREC *rec, int expected )
 	rec->r.ver.marginals = *p++;
 	rec->r.ver.completes = *p++;
 	rec->r.ver.cert_depth = *p++;
-	p += 4; /* lock flags */
+	rec->r.ver.trust_model = *p++;
+	p += 3;
 	rec->r.ver.created  = buftoulong(p); p += 4;
 	rec->r.ver.nextcheck = buftoulong(p); p += 4;
 	p += 4;
@@ -1202,6 +1221,8 @@ tdbio_read_record( ulong recnum, TRUSTREC *rec, int expected )
 	memcpy( rec->r.valid.namehash, p, 20); p+=20;
         rec->r.valid.validity = *p++;
 	rec->r.valid.next = buftoulong(p); p += 4;
+	rec->r.valid.full_count = *p++;
+	rec->r.valid.marginal_count = *p++;
 	break;
       default:
 	log_error( "%s: invalid record type %d at recnum %lu\n",
@@ -1241,7 +1262,8 @@ tdbio_write_record( TRUSTREC *rec )
 	*p++ = rec->r.ver.marginals;
 	*p++ = rec->r.ver.completes;
 	*p++ = rec->r.ver.cert_depth;
-	p += 4; /* skip lock flags */
+	*p++ = rec->r.ver.trust_model;
+	p += 3;
 	ulongtobuf(p, rec->r.ver.created); p += 4;
 	ulongtobuf(p, rec->r.ver.nextcheck); p += 4;
 	p += 4;
@@ -1281,6 +1303,8 @@ tdbio_write_record( TRUSTREC *rec )
 	memcpy( p, rec->r.valid.namehash, 20); p += 20;
 	*p++ = rec->r.valid.validity;
 	ulongtobuf( p, rec->r.valid.next); p += 4;
+	*p++ = rec->r.valid.full_count;
+	*p++ = rec->r.valid.marginal_count;
 	break;
 
       default:
