@@ -1,5 +1,6 @@
 /* trustdb.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003
+ *                                             Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -274,9 +275,8 @@ verify_own_keys(void)
                             (ulong)kid[1]);
         }
     }
-  
-  /* the --trusted-key option is again deprecated; however we automagically
-   * add those keys to the trustdb */
+
+  /* Put any --trusted-key keys into the trustdb */
   for (k = user_utk_list; k; k = k->next) 
     {
       if ( add_utk (k->kid) ) 
@@ -424,7 +424,7 @@ init_trustdb()
  * This function returns a letter for a trustvalue  Trust flags
  * are ignore.
  */
-int
+static int
 trust_letter (unsigned int value)
 {
   switch( (value & TRUST_MASK) ) 
@@ -440,7 +440,6 @@ trust_letter (unsigned int value)
     }
 }
 
-
 /****************
  * Recreate the WoT but do not ask for new ownertrusts.  Special
  * feature: In batch mode and without a forced yes, this is only done
@@ -494,7 +493,24 @@ revalidation_mark (void)
   pending_check_trustdb = 1;
 }
 
-
+void
+read_trust_options(byte *trust_model,ulong *created,ulong *nextcheck)
+{
+  TRUSTREC opts;
+
+  init_trustdb();
+
+  read_record(0,&opts,RECTYPE_VER);
+
+  if(trust_model)
+    *trust_model=opts.r.ver.trust_model;
+  if(created)
+    *created=opts.r.ver.created;
+  if(nextcheck)
+    *nextcheck=opts.r.ver.nextcheck;
+}
+
+
 /***********************************************
  ***********  Ownertrust et al. ****************
  ***********************************************/
@@ -558,7 +574,7 @@ get_ownertrust_info (PKT_public_key *pk)
     int c;
 
     otrust = get_ownertrust (pk);
-    c = trust_letter( (otrust & TRUST_MASK) );
+    c = trust_letter( otrust );
     if( !c )
 	c = '?';
     return c;
@@ -644,13 +660,19 @@ clear_ownertrust (PKT_public_key *pk)
  * Note: Caller has to do a sync 
  */
 static void
-update_validity (PKT_public_key *pk, const byte *namehash,
+update_validity (PKT_public_key *pk, PKT_user_id *uid,
                  int depth, int validity)
 {
   TRUSTREC trec, vrec;
   int rc;
   ulong recno;
-  
+  byte namehash[20];
+
+  if(uid->attrib_data)
+    rmd160_hash_buffer (namehash,uid->attrib_data,uid->attrib_len);
+  else
+    rmd160_hash_buffer (namehash, uid->name, uid->len );
+
   rc = read_trust_record (pk, &trec);
   if (rc && rc != -1)
     {
@@ -686,11 +708,13 @@ update_validity (PKT_public_key *pk, const byte *namehash,
       vrec.rectype = RECTYPE_VALID;
       memcpy (vrec.r.valid.namehash, namehash, 20);
       vrec.r.valid.next = trec.r.trust.validlist;
+      trec.r.trust.validlist = vrec.recnum;
     }
   vrec.r.valid.validity = validity;
+  vrec.r.valid.full_count = uid->help_full_count;
+  vrec.r.valid.marginal_count = uid->help_marginal_count;
   write_record (&vrec);
   trec.r.trust.depth = depth;
-  trec.r.trust.validlist = vrec.recnum;
   write_record (&trec);
 }
 
@@ -710,17 +734,18 @@ clear_validity (PKT_public_key *pk)
       tdbio_invalid ();
       return 0;
     }
-  if (rc == -1) /* no record yet - no need to clerar it then ;-) */
+  if (rc == -1) /* no record yet - no need to clear it then ;-) */
     return 0;
 
-  /* reset validity for all user IDs */
   recno = trec.r.trust.validlist;
   while (recno)
     {
       read_record (recno, &vrec, RECTYPE_VALID);
-      if ((vrec.r.valid.validity & TRUST_MASK))
+      if ((vrec.r.valid.validity & TRUST_MASK)
+	  || vrec.r.valid.marginal_count || vrec.r.valid.full_count)
         {
           vrec.r.valid.validity &= ~TRUST_MASK;
+	  vrec.r.valid.marginal_count = vrec.r.valid.full_count = 0;
           write_record (&vrec);
           any = 1;
         }
@@ -730,19 +755,53 @@ clear_validity (PKT_public_key *pk)
   return any;
 }
 
-
-
 /***********************************************
  *********  Query trustdb values  **************
  ***********************************************/
 
+/* Return true if key is disabled */
+int
+cache_disabled_value(PKT_public_key *pk)
+{
+  int rc;
+  TRUSTREC trec;
+  int disabled=0;
+
+  if(pk->is_disabled)
+    return (pk->is_disabled==2);
+
+  init_trustdb();
+
+  rc = read_trust_record (pk, &trec);
+  if (rc && rc != -1)
+    {
+      tdbio_invalid ();
+      goto leave;
+    }
+  if (rc == -1) /* no record found, so assume not disabled */
+    goto leave;
+ 
+  if(trec.r.trust.ownertrust & TRUST_FLAG_DISABLED)
+    disabled=1;
+ 
+  /* Cache it for later so we don't need to look at the trustdb every
+     time */
+  if(disabled)
+    pk->is_disabled=2;
+  else
+    pk->is_disabled=1;
+
+ leave:
+   return disabled;
+}
+
 /*
  * Return the validity information for PK.  If the namehash is not
  * NULL, the validity of the corresponsing user ID is returned,
  * otherwise, a reasonable value for the entire key is returned. 
  */
 unsigned int
-get_validity (PKT_public_key *pk, const byte *namehash)
+get_validity (PKT_public_key *pk, PKT_user_id *uid)
 {
   static int did_nextcheck;
   TRUSTREC trec, vrec;
@@ -751,7 +810,16 @@ get_validity (PKT_public_key *pk, const byte *namehash)
   unsigned int validity;
   u32 kid[2];
   PKT_public_key *main_pk;
-  
+  byte namehash[20];
+
+  if(uid)
+    {
+      if(uid->attrib_data)
+	rmd160_hash_buffer (namehash,uid->attrib_data,uid->attrib_len);
+      else
+	rmd160_hash_buffer (namehash, uid->name, uid->len );
+    }
+
   init_trustdb ();
   if (!did_nextcheck)
     {
@@ -764,7 +832,7 @@ get_validity (PKT_public_key *pk, const byte *namehash)
           if (opt.no_auto_check_trustdb) 
             {
               pending_check_trustdb = 1;
-              log_info ("please do a --check-trustdb\n");
+              log_info (_("please do a --check-trustdb\n"));
             }
           else
             {
@@ -808,18 +876,37 @@ get_validity (PKT_public_key *pk, const byte *namehash)
   while (recno)
     {
       read_record (recno, &vrec, RECTYPE_VALID);
-      if ( validity < (vrec.r.valid.validity & TRUST_MASK) )
-        validity = (vrec.r.valid.validity & TRUST_MASK);
-      if ( namehash && !memcmp (vrec.r.valid.namehash, namehash, 20) )
-        break;
+
+      if(uid)
+	{
+	  /* If a user ID is given we return the validity for that
+	     user ID ONLY.  If the namehash is not found, then there
+	     is no validity at all (i.e. the user ID wasn't
+	     signed). */
+	  if(memcmp(vrec.r.valid.namehash,namehash,20)==0)
+	    {
+	      validity=(vrec.r.valid.validity & TRUST_MASK);
+	      break;
+	    }
+	}
+      else
+	{
+	  /* If no namehash is given, we take the maximum validity
+	     over all user IDs */
+	  if ( validity < (vrec.r.valid.validity & TRUST_MASK) )
+	    validity = (vrec.r.valid.validity & TRUST_MASK);
+	}
+
       recno = vrec.r.valid.next;
     }
   
-  if (recno) /* okay, use the user ID associated one */
-    validity = (vrec.r.valid.validity & TRUST_MASK);
-
   if ( (trec.r.trust.ownertrust & TRUST_FLAG_DISABLED) )
-    validity |= TRUST_FLAG_DISABLED;
+    {
+      validity |= TRUST_FLAG_DISABLED;
+      pk->is_disabled=2;
+    }
+  else
+    pk->is_disabled=1;
 
  leave:
   /* set some flags direct from the key */
@@ -840,35 +927,68 @@ get_validity (PKT_public_key *pk, const byte *namehash)
   return validity;
 }
 
-
 int
-get_validity_info (PKT_public_key *pk, const byte *namehash)
+get_validity_info (PKT_public_key *pk, PKT_user_id *uid)
 {
     int trustlevel;
     int c;
 
-    trustlevel = get_validity (pk, namehash);
+    trustlevel = get_validity (pk, uid);
     if( trustlevel & TRUST_FLAG_DISABLED )
 	return 'd';
     if( trustlevel & TRUST_FLAG_REVOKED )
 	return 'r';
-    c = trust_letter ( (trustlevel & TRUST_MASK) );
+    c = trust_letter ( trustlevel );
     if( !c )
 	c = '?';
     return c;
 }
 
+static void
+get_validity_counts (PKT_public_key *pk, PKT_user_id *uid)
+{
+  TRUSTREC trec, vrec;
+  ulong recno;
+  byte namehash[20];
 
+  if(pk==NULL || uid==NULL)
+    BUG();
 
+  if(uid->attrib_data)
+    rmd160_hash_buffer (namehash,uid->attrib_data,uid->attrib_len);
+  else
+    rmd160_hash_buffer (namehash, uid->name, uid->len );
+
+  uid->help_marginal_count=uid->help_full_count=0;
+
+  init_trustdb ();
+
+  if(read_trust_record (pk, &trec)!=0)
+    return;
+
+  /* loop over all user IDs */
+  recno = trec.r.trust.validlist;
+  while (recno)
+    {
+      read_record (recno, &vrec, RECTYPE_VALID);
+
+      if(memcmp(vrec.r.valid.namehash,namehash,20)==0)
+	{
+	  uid->help_marginal_count=vrec.r.valid.marginal_count;
+	  uid->help_full_count=vrec.r.valid.full_count;
+	  /*  printf("Fetched marginal %d, full %d\n",uid->help_marginal_count,uid->help_full_count); */
+	  break;
+	}
+
+      recno = vrec.r.valid.next;
+    }
+}
 
 void
 list_trust_path( const char *username )
 {
 }
 
-
-
-
 /****************
  * Enumerate all keys, which are needed to build all trust paths for
  * the given key.  This function does not return the key itself or
@@ -944,7 +1064,7 @@ mark_keyblock_seen (KeyHashTable tbl, KBNODE node)
 {
   for ( ;node; node = node->next )
     if (node->pkt->pkttype == PKT_PUBLIC_KEY
-        || node->pkt->pkttype == PKT_PUBLIC_SUBKEY)
+	|| node->pkt->pkttype == PKT_PUBLIC_SUBKEY)
       {
         u32 aki[2];
 
@@ -954,7 +1074,6 @@ mark_keyblock_seen (KeyHashTable tbl, KBNODE node)
 }
 
 
-
 static void
 dump_key_array (int depth, struct key_array *keys)
 {
@@ -992,10 +1111,9 @@ dump_key_array (int depth, struct key_array *keys)
 
 
 static void
-store_validation_status (int depth, KBNODE keyblock)
+store_validation_status (int depth, KBNODE keyblock, KeyHashTable stored)
 {
   KBNODE node;
-  byte namehash[20];
   int status;
   int any = 0;
 
@@ -1004,7 +1122,6 @@ store_validation_status (int depth, KBNODE keyblock)
       if (node->pkt->pkttype == PKT_USER_ID)
         {
           PKT_user_id *uid = node->pkt->pkt.user_id;
-
           if (node->flag & 4)
             status = TRUST_FULLY;
           else if (node->flag & 2)
@@ -1016,13 +1133,11 @@ store_validation_status (int depth, KBNODE keyblock)
           
           if (status)
             {
-              if( uid->attrib_data )
-                rmd160_hash_buffer (namehash,uid->attrib_data,uid->attrib_len);
-              else
-                rmd160_hash_buffer (namehash, uid->name, uid->len );
-              
               update_validity (keyblock->pkt->pkt.public_key,
-                               namehash, depth, status);
+			       uid, depth, status);
+
+	      mark_keyblock_seen(stored,keyblock);
+
               any = 1;
             }
         }
@@ -1215,33 +1330,54 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
 {
   struct key_item *kr;
   KBNODE node, uidnode=NULL;
+  PKT_user_id *uid=NULL;
   PKT_public_key *pk = kb->pkt->pkt.public_key;
   u32 main_kid[2];
-  int issigned=0, any_signed = 0, fully_count =0, marginal_count = 0;
+  int issigned=0, any_signed = 0;
 
   keyid_from_pk(pk, main_kid);
   for (node=kb; node; node = node->next)
     {
-      if (node->pkt->pkttype == PKT_USER_ID)
+      /* A bit of discussion here: is it better for the web of trust
+	 to be built among only self-signed uids?  On the one hand, a
+	 self-signed uid is a statement that the key owner definitely
+	 intended that uid to be there, but on the other hand, a
+	 signed (but not self-signed) uid does carry trust, of a sort,
+	 even if it is a statement being made by people other than the
+	 key owner "through" the uids on the key owner's key.  I'm
+	 going with the latter.  However, if the user ID was
+	 explicitly revoked, or passively allowed to expire, that
+	 should stop validity through the user ID until it is
+	 resigned.  -dshaw */
+
+      if (node->pkt->pkttype == PKT_USER_ID
+	  && !node->pkt->pkt.user_id->is_revoked
+	  && !node->pkt->pkt.user_id->is_expired)
         {
           if (uidnode && issigned)
             {
-              if (fully_count >= opt.completes_needed
-                  || marginal_count >= opt.marginals_needed )
+              if (uid->help_full_count >= opt.completes_needed
+                  || uid->help_marginal_count >= opt.marginals_needed )
                 uidnode->flag |= 4; 
-              else if (fully_count || marginal_count)
+              else if (uid->help_full_count || uid->help_marginal_count)
                 uidnode->flag |= 2;
               uidnode->flag |= 1;
               any_signed = 1;
             }
           uidnode = node;
+	  uid=uidnode->pkt->pkt.user_id;
+
+	  /* If the selfsig is going to expire... */
+	  if(uid->expiredate && uid->expiredate<*next_expire)
+	    *next_expire = uid->expiredate;
+
           issigned = 0;
-          fully_count = marginal_count = 0;
+	  get_validity_counts(pk,uid);
           mark_usable_uid_certs (kb, uidnode, main_kid, klist, 
                                  curtime, next_expire);
         }
-      else if (node->pkt->pkttype == PKT_SIGNATURE 
-               && (node->flag & (1<<8)) )
+      else if (node->pkt->pkttype == PKT_SIGNATURE
+	       && (node->flag & (1<<8)) && uid)
         {
           PKT_signature *sig = node->pkt->pkt.signature;
           
@@ -1249,11 +1385,11 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
           if (kr)
             {
               if (kr->ownertrust == TRUST_ULTIMATE)
-                fully_count = opt.completes_needed;
+                uid->help_full_count = opt.completes_needed;
               else if (kr->ownertrust == TRUST_FULLY)
-                fully_count++;
+                uid->help_full_count++;
               else if (kr->ownertrust == TRUST_MARGINAL)
-                marginal_count++;
+                uid->help_marginal_count++;
               issigned = 1;
             }
         }
@@ -1261,10 +1397,10 @@ validate_one_keyblock (KBNODE kb, struct key_item *klist,
 
   if (uidnode && issigned)
     {
-      if (fully_count >= opt.completes_needed
-               || marginal_count >= opt.marginals_needed )
+      if (uid->help_full_count >= opt.completes_needed
+	  || uid->help_marginal_count >= opt.marginals_needed )
         uidnode->flag |= 4; 
-      else if (fully_count || marginal_count)
+      else if (uid->help_full_count || uid->help_marginal_count)
         uidnode->flag |= 2;
       uidnode->flag |= 1;
       any_signed = 1;
@@ -1289,7 +1425,7 @@ search_skipfnc (void *opaque, u32 *kid)
  * Caller hast to release the returned array.  
  */
 static struct key_array *
-validate_key_list (KEYDB_HANDLE hd, KeyHashTable visited,
+validate_key_list (KEYDB_HANDLE hd, KeyHashTable full_trust,
                    struct key_item *klist, u32 curtime, u32 *next_expire)
 {
   KBNODE keyblock = NULL;
@@ -1313,7 +1449,7 @@ validate_key_list (KEYDB_HANDLE hd, KeyHashTable visited,
   memset (&desc, 0, sizeof desc);
   desc.mode = KEYDB_SEARCH_MODE_FIRST;
   desc.skipfnc = search_skipfnc;
-  desc.skipfncvalue = visited;
+  desc.skipfncvalue = full_trust;
   rc = keydb_search (hd, &desc, 1);
   if (rc == -1)
     {
@@ -1356,10 +1492,12 @@ validate_key_list (KEYDB_HANDLE hd, KeyHashTable visited,
       if (pk->has_expired || pk->is_revoked)
         {
           /* it does not make sense to look further at those keys */
-          mark_keyblock_seen (visited, keyblock);
+          mark_keyblock_seen (full_trust, keyblock);
         }
       else if (validate_one_keyblock (keyblock, klist, curtime, next_expire))
         {
+	  KBNODE node;
+
           if (pk->expiredate && pk->expiredate >= curtime
               && pk->expiredate < *next_expire)
             *next_expire = pk->expiredate;
@@ -1369,8 +1507,17 @@ validate_key_list (KEYDB_HANDLE hd, KeyHashTable visited,
             keys = m_realloc (keys, (maxkeys+1) * sizeof *keys);
           }
           keys[nkeys++].keyblock = keyblock;
-          /* this key is signed - don't check it again */
-          mark_keyblock_seen (visited, keyblock);
+
+	  /* Optimization - if all uids are fully trusted, then we
+	     never need to consider this key as a candidate again. */
+
+	  for (node=keyblock; node; node = node->next)
+	    if (node->pkt->pkttype == PKT_USER_ID && !(node->flag & 4))
+	      break;
+
+	  if(node==NULL)
+	    mark_keyblock_seen (full_trust, keyblock);
+
           keyblock = NULL;
         }
 
@@ -1389,9 +1536,9 @@ validate_key_list (KEYDB_HANDLE hd, KeyHashTable visited,
   return keys;
 } 
 
-
+/* Caller must sync */
 static void
-reset_unconnected_keys (KEYDB_HANDLE hd, KeyHashTable visited)
+reset_trust_records (KEYDB_HANDLE hd, KeyHashTable exclude)
 {
   int rc;
   KBNODE keyblock = NULL;
@@ -1407,8 +1554,11 @@ reset_unconnected_keys (KEYDB_HANDLE hd, KeyHashTable visited)
 
   memset (&desc, 0, sizeof desc);
   desc.mode = KEYDB_SEARCH_MODE_FIRST;
-  desc.skipfnc = search_skipfnc;
-  desc.skipfncvalue = visited;
+  if(exclude)
+    {
+      desc.skipfnc = search_skipfnc;
+      desc.skipfncvalue = exclude;
+    }
   rc = keydb_search (hd, &desc, 1);
   if (rc && rc != -1 )
     log_error ("keydb_search_first failed: %s\n", g10_errstr(rc));
@@ -1436,11 +1586,9 @@ reset_unconnected_keys (KEYDB_HANDLE hd, KeyHashTable visited)
         log_error ("keydb_search_next failed: %s\n", g10_errstr(rc));
     }
   if (opt.verbose)
-    log_info ("%d unconnected keys (%d trust records cleared)\n",
+    log_info (_("%d keys processed (%d validity counts cleared)\n"),
               count, nreset);
-  do_sync ();
-} 
-
+}
 
 /*
  * Run the key validation procedure.
@@ -1480,22 +1628,28 @@ validate_keys (int interactive)
   int depth;
   int key_count;
   int ot_unknown, ot_undefined, ot_never, ot_marginal, ot_full, ot_ultimate;
-  KeyHashTable visited;
+  KeyHashTable stored,used,full_trust;
   u32 start_time, next_expire;
 
   start_time = make_timestamp ();
   next_expire = 0xffffffff; /* set next expire to the year 2106 */
-  visited = new_key_hash_table ();
+  stored = new_key_hash_table ();
+  used = new_key_hash_table ();
+  full_trust = new_key_hash_table ();
   /* Fixme: Instead of always building a UTK list, we could just build it
    * here when needed */
   if (!utk_list)
     {
-      log_info ("no ultimately trusted keys found\n");
+      log_info (_("no ultimately trusted keys found\n"));
       goto leave;
     }
 
+  kdb = keydb_new (0);
 
-  /* mark all UTKs as visited and set validity to ultimate */
+  reset_trust_records (kdb,NULL);
+
+  /* mark all UTKs as used and fully_trusted and set validity to
+     ultimate */
   for (k=utk_list; k; k = k->next)
     {
       KBNODE keyblock;
@@ -1508,21 +1662,14 @@ validate_keys (int interactive)
                        " trusted key %08lX not found\n"), (ulong)k->kid[1]);
           continue;
         }
-      mark_keyblock_seen (visited, keyblock);
+      mark_keyblock_seen (used, keyblock);
+      mark_keyblock_seen (stored, keyblock);
+      mark_keyblock_seen (full_trust, keyblock);
       pk = keyblock->pkt->pkt.public_key;
       for (node=keyblock; node; node = node->next)
         {
           if (node->pkt->pkttype == PKT_USER_ID)
-            {
-              byte namehash[20];
-              PKT_user_id *uid = node->pkt->pkt.user_id;
-              
-              if( uid->attrib_data )
-                rmd160_hash_buffer (namehash,uid->attrib_data,uid->attrib_len);
-              else
-                rmd160_hash_buffer (namehash, uid->name, uid->len );
-              update_validity (pk, namehash, 0, TRUST_ULTIMATE);
-            }
+	    update_validity (pk, node->pkt->pkt.user_id, 0, TRUST_ULTIMATE);
         }
       if ( pk->expiredate && pk->expiredate >= start_time
            && pk->expiredate < next_expire)
@@ -1532,9 +1679,7 @@ validate_keys (int interactive)
       do_sync ();
     }
 
-
   klist = utk_list;
-  kdb = keydb_new (0);
 
   for (depth=0; depth < opt.max_cert_depth; depth++)
     {
@@ -1566,7 +1711,8 @@ validate_keys (int interactive)
         }
 
       /* Find all keys which are signed by a key in kdlist */
-      keys = validate_key_list (kdb, visited, klist, start_time, &next_expire);
+      keys = validate_key_list (kdb, full_trust, klist,
+				start_time, &next_expire);
       if (!keys) 
         {
           log_error ("validate_key_list failed\n");
@@ -1574,7 +1720,6 @@ validate_keys (int interactive)
           goto leave;
         }
 
-
       for (key_count=0, kar=keys; kar->keyblock; kar++, key_count++)
         ;
 
@@ -1582,14 +1727,17 @@ validate_keys (int interactive)
       if (opt.verbose > 1)
         dump_key_array (depth, keys);
 
+      for (kar=keys; kar->keyblock; kar++)
+          store_validation_status (depth, kar->keyblock, stored);
+
+      /* This should be valid=%d now, but I'm not changing it so I
+	 don't break the translated strings in the stable branch.
+	 Change it in devel. -dms */
       log_info (_("checking at depth %d signed=%d"
                   " ot(-/q/n/m/f/u)=%d/%d/%d/%d/%d/%d\n"), 
                 depth, key_count, ot_unknown, ot_undefined,
                 ot_never, ot_marginal, ot_full, ot_ultimate ); 
 
-      for (kar=keys; kar->keyblock; kar++)
-          store_validation_status (depth, kar->keyblock);
-
       /* Build a new kdlist from all fully valid keys in KEYS */
       if (klist != utk_list)
         release_key_items (klist);
@@ -1600,29 +1748,43 @@ validate_keys (int interactive)
             {
               if (node->pkt->pkttype == PKT_USER_ID && (node->flag & 4))
                 {
-                  k = new_key_item ();
-                  keyid_from_pk (kar->keyblock->pkt->pkt.public_key, k->kid);
-                  k->ownertrust = get_ownertrust (kar->keyblock
-                                                  ->pkt->pkt.public_key);
-                  k->next = klist;
-                  klist = k;
-                  break;
-                }
-            }
-        }
+		  u32 kid[2];
+
+		  /* have we used this key already? */
+                  keyid_from_pk (kar->keyblock->pkt->pkt.public_key, kid);
+		  if(test_key_hash_table(used,kid)==0)
+		    {
+		      /* Normally we add both the primary and subkey
+			 ids to the hash via mark_keyblock_seen, but
+			 since we aren't using this hash as a skipfnc,
+			 that doesn't matter here. */
+		      add_key_hash_table (used,kid);
+		      k = new_key_item ();
+		      k->kid[0]=kid[0];
+		      k->kid[1]=kid[1];
+		      k->ownertrust = (get_ownertrust (kar->keyblock
+						       ->pkt->pkt.public_key)
+				       & TRUST_MASK);
+		      k->next = klist;
+		      klist = k;
+		      break;
+		    }
+		}
+	    }
+	}
       release_key_array (keys);
       keys = NULL;
       if (!klist)
         break; /* no need to dive in deeper */
     }
 
-  reset_unconnected_keys (kdb, visited);
-
  leave:
   keydb_release (kdb);
   release_key_array (keys);
   release_key_items (klist);
-  release_key_hash_table (visited);
+  release_key_hash_table (full_trust);
+  release_key_hash_table (used);
+  release_key_hash_table (stored);
   if (!rc && !quit) /* mark trustDB as checked */
     {
       if (next_expire == 0xffffffff || next_expire < start_time )
@@ -1636,7 +1798,6 @@ validate_keys (int interactive)
       do_sync ();
       pending_check_trustdb = 0;
     }
+
   return rc;
 }
-
-
