@@ -20,8 +20,8 @@
 #ifndef G10_UTIL_H
 #define G10_UTIL_H
 
-#if defined (__MINGW32__) || defined (__CYGWIN32__)
-# include <stdarg.h>
+#if defined (_WIN32) || defined (__CYGWIN32__)
+#include <stdarg.h>
 #endif
 
 #include "types.h"
@@ -91,9 +91,9 @@ void g10_log_hexdump( const char *text, const char *buf, size_t len );
   void g10_log_debug_f( const char *fname,  const char *fmt, ... )
 			    __attribute__ ((format (printf,2,3)));
 #ifndef __riscos__
-  #define BUG() g10_log_bug0(  __FILE__ , __LINE__, __FUNCTION__ )
+#define BUG() g10_log_bug0(  __FILE__ , __LINE__, __FUNCTION__ )
 #else
-  #define BUG() g10_log_bug0(  __FILE__ , __LINE__, __func__ )
+#define BUG() g10_log_bug0(  __FILE__ , __LINE__, __func__ )
 #endif
 #else
   void g10_log_bug( const char *fmt, ... );
@@ -106,7 +106,7 @@ void g10_log_hexdump( const char *text, const char *buf, size_t len );
   void g10_log_error_f( const char *fname, const char *fmt, ... );
   void g10_log_info_f( const char *fname, const char *fmt, ... );
   void g10_log_debug_f( const char *fname, const char *fmt, ... );
-  #define BUG() g10_log_bug0( __FILE__ , __LINE__ )
+#define BUG() g10_log_bug0( __FILE__ , __LINE__ )
 #endif
 
 #define log_hexdump g10_log_hexdump
@@ -223,13 +223,13 @@ int strcasecmp( const char *, const char *b);
 int strncasecmp (const char *, const char *b, size_t n);
 #endif
 #ifndef HAVE_STRTOUL
-  #define strtoul(a,b,c)  ((unsigned long)strtol((a),(b),(c)))
+#define strtoul(a,b,c)  ((unsigned long)strtol((a),(b),(c)))
 #endif
 #ifndef HAVE_MEMMOVE
-  #define memmove(d, s, n) bcopy((s), (d), (n))
+#define memmove(d, s, n) bcopy((s), (d), (n))
 #endif
 
-#if defined (__MINGW32__)
+#if defined (_WIN32)
 /*-- w32reg.c --*/
 char *read_w32_registry_string( const char *root,
 				const char *dir, const char *name );
@@ -242,16 +242,16 @@ int vasprintf ( char **result, const char *format, va_list args);
 
 /**** other missing stuff ****/
 #ifndef HAVE_ATEXIT  /* For SunOS */
-  #define atexit(a)    (on_exit((a),0))
+#define atexit(a)    (on_exit((a),0))
 #endif
 
 #ifndef HAVE_RAISE
-  #define raise(a) kill(getpid(), (a))
+#define raise(a) kill(getpid(), (a))
 #endif
 
 /******** some macros ************/
 #ifndef STR
-  #define STR(v) #v
+#define STR(v) #v
 #endif
 #define STR2(v) STR(v)
 #define DIM(v) (sizeof(v)/sizeof((v)[0]))
@@ -260,6 +260,16 @@ int vasprintf ( char **result, const char *format, va_list args);
 #define wipememory2(_ptr,_set,_len) do { volatile char *_vptr=(volatile char *)(_ptr); size_t _vlen=(_len); while(_vlen) { *_vptr=(_set); _vptr++; _vlen--; } } while(0)
 #define wipememory(_ptr,_len) wipememory2(_ptr,0,_len)
 
+/*-- macros to replace ctype ones and avoid locale problems --*/
+#define spacep(p)   (*(p) == ' ' || *(p) == '\t')
+#define digitp(p)   (*(p) >= '0' && *(p) <= '9')
+#define hexdigitp(a) (digitp (a)                     \
+                      || (*(a) >= 'A' && *(a) <= 'F')  \
+                      || (*(a) >= 'a' && *(a) <= 'f'))
+/* Note this isn't identical to a C locale isspace() without \f and
+   \v, but works for the purposes used here. */
+#define ascii_isspace(a) ((a)==' ' || (a)=='\n' || (a)=='\r' || (a)=='\t')
+
 /******* RISC OS stuff ***********/
 #ifdef __riscos__
 /* needed for strcasecmp() */
@@ -285,11 +295,10 @@ void dump_fdlist(void);
 void list_openfiles(void);
 #endif
 #ifndef __RISCOS__C__
-  #define getpid riscos_getpid
-  #define kill(a,b) riscos_kill((a),(b))
-  #define access(a,b) riscos_access((a),(b))
+#define getpid riscos_getpid
+#define kill(a,b) riscos_kill((a),(b))
+#define access(a,b) riscos_access((a),(b))
 #endif /* !__RISCOS__C__ */
 #endif /* __riscos__ */
 
 #endif /*G10_UTIL_H*/
-
